@@ -26,9 +26,15 @@
  */
 #include <string.h>
 
-#include <qpixmap.h>
-#include <qprocess.h>
+#include <QPixmap>
+#include <QProcess>
 #include <qsound.h>
+#include <QDir>
+#include <QFile>
+#include <QUrl>
+#include <QVariant>
+#include <QTextDocument>
+#include <QDesktopServices>
 
 #include <dclib/core/cstring.h>
 #include <dclib/core/cxml.h>
@@ -131,6 +137,7 @@
 #define XML_BACKGROUNDPIXMAP		"backgroundpixmap"
 #define XML_BROWSER			"browser"
 #define XML_CUSTOM_BROWSER		"custom_browser"
+
 #define XML_SHOWCHATINTAB		"showchatintab"
 #define XML_OPENCLIENTWINDOWS		"openclientwindows"
 
@@ -255,12 +262,12 @@ DCConfig::DCConfig( CString configpath ) : CConfig(configpath)
 	// gui defaults
 	bOpenPrivateChatWindow	= true;
 	m_eUnit			= euAUTO;
-	sTheme			= "";
 #if defined(Q_OS_MACX)
-	sThemeQT4		= "Mac";
+	sTheme			= "Mac";
 #else
-	sThemeQT4		= "";
+	sTheme			= "";
 #endif
+	sThemeQT3		= "";
 	sLanguageFile		= "";
 	sAppFont		= DEFAULT_APP_FONT;
 	
@@ -344,9 +351,7 @@ DCConfig::DCConfig( CString configpath ) : CConfig(configpath)
 	iFloodCount    = 3;
 	m_bFloodOpKick = false;
 	
-	m_pMimeSourceFactory = 0;
-
-	pProcess = new QProcess ();
+	pProcess = new QProcess();
 
 	iDefaultDownloadMode = 0;
 
@@ -362,9 +367,7 @@ DCConfig::DCConfig( CString configpath ) : CConfig(configpath)
 #else
 	sBrowser = "xdg-open";
 #endif
-	/*
-	 * false to always use default above
-	 */
+	/* false to use QDesktopServices */
 	m_bCustomBrowser = false;
 	bShowChatInTab = true;
 	m_nChatMaxParagraph = 0;
@@ -398,7 +401,7 @@ DCConfig::DCConfig( CString configpath ) : CConfig(configpath)
 	m_sChatColor_17 = "#0000ff";
 	m_sChatColor_18 = "#ff0c52";
 	m_sChatColor_19 = "#000000";
-
+	
 	/* default false to use theme color */
 	m_bChatBackgroundColorEnabled = false;
 	/* white */
@@ -433,7 +436,7 @@ DCConfig::DCConfig( CString configpath ) : CConfig(configpath)
 	m_bAdjustSearchResultHubNames = false;
 	m_bRemovePublicHubEnabled = false;
 
-	m_sQT4MainWindowState = "";
+	m_baQMainWindowState = "";
 	
 	m_nTabCompleteMatchMode = 0;
 	m_bTabCompleteStripTags = false;
@@ -451,9 +454,6 @@ DCConfig::DCConfig( CString configpath ) : CConfig(configpath)
 	
 	m_eMagnetActionChat = eMagnetPrompt;
 
-	m_lUserChatCommands.setAutoDelete(true);
-	m_lAutoResponses.setAutoDelete(true);
-
 #if defined(Q_OS_MACX) 
 	CFURLRef pluginRef = CFBundleCopyBundleURL(CFBundleGetMainBundle()); 
 	CFStringRef macPath = CFURLCopyFileSystemPath(pluginRef, kCFURLPOSIXPathStyle); 
@@ -477,14 +477,12 @@ DCConfig::~DCConfig()
 	
 	delete pProcess;
 
-	if ( m_pMimeSourceFactory )
-	{
-		QMimeSourceFactory::removeFactory(m_pMimeSourceFactory);
-		delete m_pMimeSourceFactory;
-	}
-
 	if ( m_pEmoticons )
 	{
+		for ( QList<DC_EmoticonObject*>::const_iterator it = m_pEmoticons->constBegin(); it != m_pEmoticons->constEnd(); ++it )
+		{
+			delete *it;
+		}
 		m_pEmoticons->clear();
 		delete m_pEmoticons;
 		m_pEmoticons = 0;
@@ -493,15 +491,13 @@ DCConfig::~DCConfig()
 	// delete the maps, fix memory leak on shutdown
 	for ( SettingMap::const_iterator it = m_guiSettingMap.constBegin(); it != m_guiSettingMap.constEnd(); ++it )
 	{
-		delete it.data();
+		delete it.value();
 	}
 	m_guiSettingMap.clear();
 
-	m_EmoticonImage.reset();
-	
 	for ( QMap<int, DC_UserMenuCommand*>::const_iterator it = m_UserMenuCommands.constBegin(); it != m_UserMenuCommands.constEnd(); ++it )
 	{
-		delete it.data();
+		delete it.value();
 	}
 	m_UserMenuCommands.clear();
 	
@@ -517,7 +513,16 @@ DCConfig::~DCConfig()
 		}
 	}
 	
+	for ( QList<DC_UserChatCommand*>::const_iterator it = m_lUserChatCommands.constBegin(); it != m_lUserChatCommands.constEnd(); ++it )
+	{
+		delete *it;
+	}
 	m_lUserChatCommands.clear();
+	
+	for ( QList<DC_AutoResponseObject*>::const_iterator it = m_lAutoResponses.constBegin(); it != m_lAutoResponses.constEnd(); ++it )
+	{
+		delete *it;
+	}
 	m_lAutoResponses.clear();
 }
 
@@ -568,10 +573,10 @@ void DCConfig::ParseDCGuiConfig( CXml * xml )
 					else if ( xml_tmp == XML_UNIT_GBYTE )
 						m_eUnit = euGBYTE;
 				}
-				else if ( xml_name == XML_THEME )
-					sTheme = xml->Content().Data();
 				else if ( xml_name == XML_THEME_QT4 )
-					sThemeQT4 = xml->Content().Data();
+					sTheme = xml->Content().Data();
+				else if ( xml_name == XML_THEME )
+					sThemeQT3 = xml->Content().Data();
 				else if ( xml_name == XML_BACKGROUNDPIXMAP )
 					sBackgroundPixmap = xml->Content().Data();
 				else if ( xml_name == XML_BROWSER )
@@ -888,7 +893,7 @@ void DCConfig::ParseDCGuiConfig( CXml * xml )
 				else if ( xml_name == XML_FILELIST_NAMING_SCHEME )
 					m_nFilelistNamingSchemeVersion = xml->Content().asINT();
 				else if ( xml_name == XML_QMAINWINDOW_STATE )
-					m_sQT4MainWindowState = xml->Content().Data();
+					m_baQMainWindowState = QByteArray::fromBase64(xml->Content().Data());
 				else if ( xml_name == XML_FOLDERS_RIGHT_PANE )
 					SetFoldersInRightPane( xml->GetBoolChild() );
 				else if ( xml_name == XML_SIZES_IN_LEFT_PANE )
@@ -978,10 +983,10 @@ int DCConfig::LoadDCGui()
 	}
 
 	/* load user list icons, theme either from config file or default */
- 	QString actualtheme = m_sUserListIconTheme;
+	QString actualtheme = m_sUserListIconTheme;
 	m_sUserListIconTheme += "forceload";
 	SetUserListIconTheme( actualtheme );
- 
+
 	InitMimeSourceFactory();
 
 	return err;
@@ -993,6 +998,7 @@ int DCConfig::SaveDCGui()
 	int err=0;
 	CString s;
 	CXml xml;
+
 	xml.NewDoc( XML_DCGUI_CONFIG );
 
 	// gui
@@ -1009,7 +1015,7 @@ int DCConfig::SaveDCGui()
 	xml.NewBoolChild( XML_SHOWCHATINTAB, bShowChatInTab );
 	xml.NewNumericChild( XML_CHATMAXPARAGRAPH, m_nChatMaxParagraph );
 
-	xml.NewStringChild( XML_USER_PHOTO, m_sUserPhotoFileName.ascii() );
+	xml.NewStringChild( XML_USER_PHOTO, m_sUserPhotoFileName.toAscii().constData() );
 
 	switch(m_eUnit)
 	{
@@ -1031,21 +1037,21 @@ int DCConfig::SaveDCGui()
 	}
 
 	xml.NewStringChild( XML_UNIT, s );
-	xml.NewStringChild( XML_THEME, sTheme.ascii() );
-	xml.NewStringChild( XML_THEME_QT4, sThemeQT4.ascii() );
-	xml.NewStringChild( XML_BACKGROUNDPIXMAP, sBackgroundPixmap.ascii() );
-	xml.NewStringChild( XML_BROWSER, sBrowser.ascii() );
+	xml.NewStringChild( XML_THEME, sThemeQT3.toAscii().constData() );
+	xml.NewStringChild( XML_THEME_QT4, sTheme.toAscii().constData() );
+	xml.NewStringChild( XML_BACKGROUNDPIXMAP, sBackgroundPixmap.toAscii().constData() );
+	xml.NewStringChild( XML_BROWSER, sBrowser.toAscii().constData() );
 	xml.NewBoolChild( XML_CUSTOM_BROWSER, m_bCustomBrowser );
 	xml.NewNumericChild( XML_SENDCHATINDEX, m_nSendChat );
-	xml.NewStringChild( XML_LANGUAGEFILE, sLanguageFile.ascii() );
-	xml.NewStringChild( XML_APP_FONT, sAppFont.ascii() );
+	xml.NewStringChild( XML_LANGUAGEFILE, sLanguageFile.toAscii().constData() );
+	xml.NewStringChild( XML_APP_FONT, sAppFont.toAscii().constData() );
 	xml.NewBoolChild( XML_USE_CUSTOM_FONT, m_bUseCustomFont );
 	xml.NewNumericChild( XML_APP_WIDTH, iAppWidth );
 	xml.NewNumericChild( XML_APP_HEIGHT, iAppHeight );
 	xml.NewBoolChild( XML_ENABLETRAY, m_bEnableTray );
 	xml.NewBoolChild( XML_QUERYONEXIT, bQueryOnExit );
 	xml.NewBoolChild( XML_QUERYONFILEDELETE, bQueryOnFileDelete );
-	xml.NewStringChild( XML_QMAINWINDOW_STATE, m_sQT4MainWindowState.ascii() );
+	xml.NewStringChild( XML_QMAINWINDOW_STATE, m_baQMainWindowState.toBase64().constData() );
 	xml.NewBoolChild( XML_FOLDERS_RIGHT_PANE, m_bFoldersInRightPane );
 	xml.NewBoolChild( XML_SIZES_IN_LEFT_PANE, m_bFolderSizesInLeftPane );
 	xml.NewBoolChild( XML_FOLDERS_RIGHT_PANE_ON_TOP, m_bFoldersInRightPaneOnTop );
@@ -1056,13 +1062,13 @@ int DCConfig::SaveDCGui()
 	xml.NewBoolChild( XML_TAB_COMPLETE_STRIP, m_bTabCompleteStripTags );
 
 	// sound settings
-	xml.NewStringChild( XML_SOUND_FILE_CONNECT, sSoundFileConnect.ascii() );
-	xml.NewStringChild( XML_SOUND_FILE_DISCONNECT, sSoundFileDisconnect.ascii() );
-	xml.NewStringChild( XML_SOUND_FILE_SEND, sSoundFileSend.ascii() );
-	xml.NewStringChild( XML_SOUND_FILE_RECEIVE, sSoundFileReceive.ascii() );
-	xml.NewStringChild( XML_SOUND_FILE_FIRSTRECEIVE, sSoundFileFirstReceive.ascii() );
-	xml.NewStringChild( XML_SOUND_FILE_NICK_MENTIONED, sSoundFileNickMentioned.ascii() );
-	xml.NewStringChild( XML_EXTERNAL_PLAYER, sExternalPlayer.ascii() );
+	xml.NewStringChild( XML_SOUND_FILE_CONNECT, sSoundFileConnect.toAscii().constData() );
+	xml.NewStringChild( XML_SOUND_FILE_DISCONNECT, sSoundFileDisconnect.toAscii().constData() );
+	xml.NewStringChild( XML_SOUND_FILE_SEND, sSoundFileSend.toAscii().constData() );
+	xml.NewStringChild( XML_SOUND_FILE_RECEIVE, sSoundFileReceive.toAscii().constData() );
+	xml.NewStringChild( XML_SOUND_FILE_FIRSTRECEIVE, sSoundFileFirstReceive.toAscii().constData() );
+	xml.NewStringChild( XML_SOUND_FILE_NICK_MENTIONED, sSoundFileNickMentioned.toAscii().constData() );
+	xml.NewStringChild( XML_EXTERNAL_PLAYER, sExternalPlayer.toAscii().constData() );
 	xml.NewBoolChild( XML_SOUND_ENABLED_CONNECT, bSoundEnabledConnect );
 	xml.NewBoolChild( XML_SOUND_ENABLED_DISCONNECT, bSoundEnabledDisconnect );
 	xml.NewBoolChild( XML_SOUND_ENABLED_SEND, bSoundEnabledSend );
@@ -1088,34 +1094,34 @@ int DCConfig::SaveDCGui()
 
 	for ( QStringList::const_iterator slit = m_LogChatNickNameFilter.constBegin(); slit != m_LogChatNickNameFilter.constEnd(); ++slit )
 	{
-		xml.NewStringChild( "NICKNAMEFILTER", (*slit).ascii() );
+		xml.NewStringChild( "NICKNAMEFILTER", (*slit).toAscii().constData() );
 	}
 
 	// valknut data path
-	xml.NewStringChild( XML_VALKNUT_DATA_PATH, m_sValknutDataPath.ascii() );
+	xml.NewStringChild( XML_VALKNUT_DATA_PATH, m_sValknutDataPath.toAscii().constData() );
 
-	xml.NewStringChild( XML_CC_CHATTIMESTAMP, GetChatColor(eccCHATTIMESTAMP).ascii() );
-	xml.NewStringChild( XML_CC_CHATLOCALNICK, GetChatColor(eccCHATLOCALNICK).ascii() );
-	xml.NewStringChild( XML_CC_CHATLOCALTEXT, GetChatColor(eccCHATLOCALTEXT).ascii() );
-	xml.NewStringChild( XML_CC_CHATREMOTENICK, GetChatColor(eccCHATREMOTENICK).ascii() );
-	xml.NewStringChild( XML_CC_CHATREMOTETEXT, GetChatColor(eccCHATREMOTETEXT).ascii() );
-	xml.NewStringChild( XML_CC_CHATSTATUSNICK, GetChatColor(eccCHATSTATUSNICK).ascii() );
-	xml.NewStringChild( XML_CC_CHATSTATUSTEXT, GetChatColor(eccCHATSTATUSTEXT).ascii() );
-	xml.NewStringChild( XML_CC_PUBLICCHATTIMESTAMP, GetChatColor(eccPUBLICCHATTIMESTAMP).ascii() );
-	xml.NewStringChild( XML_CC_PUBLICCHATLOCALNICK, GetChatColor(eccPUBLICCHATLOCALNICK).ascii() );
-	xml.NewStringChild( XML_CC_PUBLICCHATLOCALTEXT, GetChatColor(eccPUBLICCHATLOCALTEXT).ascii() );
-	xml.NewStringChild( XML_CC_PUBLICCHATREMOTENICK, GetChatColor(eccPUBLICCHATREMOTENICK).ascii() );
-	xml.NewStringChild( XML_CC_PUBLICCHATREMOTETEXT, GetChatColor(eccPUBLICCHATREMOTETEXT).ascii() );
-	xml.NewStringChild( XML_CC_PUBLICCHATSTATUSNICK, GetChatColor(eccPUBLICCHATSTATUSNICK).ascii() );
-	xml.NewStringChild( XML_CC_PUBLICCHATSTATUSTEXT, GetChatColor(eccPUBLICCHATSTATUSTEXT).ascii() );
-	xml.NewStringChild( XML_CC_PUBLICPRIVATECHATNICK, GetChatColor(eccPUBLICPRIVATECHATNICK).ascii() );
-	xml.NewStringChild( XML_CC_PUBLICPRIVATECHATTEXT, GetChatColor(eccPUBLICPRIVATECHATTEXT).ascii() );
-	xml.NewStringChild( XML_CC_CHATSAY, GetChatColor(eccCHATSAY).ascii() );
-	xml.NewStringChild( XML_CC_PUBLICCHATMENICK, GetChatColor(eccPUBLICCHATMENICK).ascii() );
-	xml.NewStringChild( XML_CC_OPNICK, GetChatColor(eccOPNICK).ascii() );
+	xml.NewStringChild( XML_CC_CHATTIMESTAMP, GetChatColor(eccCHATTIMESTAMP).toAscii().constData() );
+	xml.NewStringChild( XML_CC_CHATLOCALNICK, GetChatColor(eccCHATLOCALNICK).toAscii().constData() );
+	xml.NewStringChild( XML_CC_CHATLOCALTEXT, GetChatColor(eccCHATLOCALTEXT).toAscii().constData() );
+	xml.NewStringChild( XML_CC_CHATREMOTENICK, GetChatColor(eccCHATREMOTENICK).toAscii().constData() );
+	xml.NewStringChild( XML_CC_CHATREMOTETEXT, GetChatColor(eccCHATREMOTETEXT).toAscii().constData() );
+	xml.NewStringChild( XML_CC_CHATSTATUSNICK, GetChatColor(eccCHATSTATUSNICK).toAscii().constData() );
+	xml.NewStringChild( XML_CC_CHATSTATUSTEXT, GetChatColor(eccCHATSTATUSTEXT).toAscii().constData() );
+	xml.NewStringChild( XML_CC_PUBLICCHATTIMESTAMP, GetChatColor(eccPUBLICCHATTIMESTAMP).toAscii().constData() );
+	xml.NewStringChild( XML_CC_PUBLICCHATLOCALNICK, GetChatColor(eccPUBLICCHATLOCALNICK).toAscii().constData() );
+	xml.NewStringChild( XML_CC_PUBLICCHATLOCALTEXT, GetChatColor(eccPUBLICCHATLOCALTEXT).toAscii().constData() );
+	xml.NewStringChild( XML_CC_PUBLICCHATREMOTENICK, GetChatColor(eccPUBLICCHATREMOTENICK).toAscii().constData() );
+	xml.NewStringChild( XML_CC_PUBLICCHATREMOTETEXT, GetChatColor(eccPUBLICCHATREMOTETEXT).toAscii().constData() );
+	xml.NewStringChild( XML_CC_PUBLICCHATSTATUSNICK, GetChatColor(eccPUBLICCHATSTATUSNICK).toAscii().constData() );
+	xml.NewStringChild( XML_CC_PUBLICCHATSTATUSTEXT, GetChatColor(eccPUBLICCHATSTATUSTEXT).toAscii().constData() );
+	xml.NewStringChild( XML_CC_PUBLICPRIVATECHATNICK, GetChatColor(eccPUBLICPRIVATECHATNICK).toAscii().constData() );
+	xml.NewStringChild( XML_CC_PUBLICPRIVATECHATTEXT, GetChatColor(eccPUBLICPRIVATECHATTEXT).toAscii().constData() );
+	xml.NewStringChild( XML_CC_CHATSAY, GetChatColor(eccCHATSAY).toAscii().constData() );
+	xml.NewStringChild( XML_CC_PUBLICCHATMENICK, GetChatColor(eccPUBLICCHATMENICK).toAscii().constData() );
+	xml.NewStringChild( XML_CC_OPNICK, GetChatColor(eccOPNICK).toAscii().constData() );
 	
 	xml.NewBoolChild( XML_CHAT_BG_COLOR_ENABLED, m_bChatBackgroundColorEnabled );
-	xml.NewStringChild( XML_CHAT_BG_COLOR, m_sChatBackgroundColor.ascii() );
+	xml.NewStringChild( XML_CHAT_BG_COLOR, m_sChatBackgroundColor.toAscii().constData() );
 
 	// transfer view settings
 	xml.NewBoolChild( XML_TV_CHUNKPERCENT, bTVChunkPercent );
@@ -1139,9 +1145,9 @@ int DCConfig::SaveDCGui()
 	xml.NewNumericChild( XML_AUTO_AWAY_TIME, m_nAutoAwayTime );
 	xml.NewBoolChild( XML_CHAT_SHOW_JOINS_AND_PARTS, m_bChatShowJoinsAndParts );
 	xml.NewBoolChild( XML_CHAT_SHOW_JOINS_AND_PARTS_ONLY_FAV, m_bChatShowJoinsAndPartsOnlyFav );
-	xml.NewStringChild( XML_CHAT_SUPPRESSED_NICKS, m_sSuppressedNicks.ascii() );
+	xml.NewStringChild( XML_CHAT_SUPPRESSED_NICKS, m_sSuppressedNicks.toAscii().constData() );
 	xml.NewBoolChild( XML_SEND_UNKNOWN_COMMANDS, m_bSendUnknownCommandsAsChat );
-	xml.NewStringChild( XML_ALLOWED_UNKNOWN_COMMANDS, m_sAllowedUnknownCommands.ascii() );
+	xml.NewStringChild( XML_ALLOWED_UNKNOWN_COMMANDS, m_sAllowedUnknownCommands.toAscii().constData() );
 	xml.NewNumericChild( XML_DOUBLE_CLICK_ACTION, m_nDoubleClickAction );
 	xml.NewNumericChild( XML_MAGNET_ACTION_CHAT, m_eMagnetActionChat );
 
@@ -1157,17 +1163,17 @@ int DCConfig::SaveDCGui()
 	
 	xml.NewBoolChild( XML_USER_COMMAND_SUBMENU, m_bUserCommandSubmenu );
 	// user menu commands
-	DC_UserMenuCommand * umc;
+	DC_UserMenuCommand * umc = 0;
 	for ( QMap<int, DC_UserMenuCommand*>::const_iterator umcit = m_UserMenuCommands.constBegin(); umcit != m_UserMenuCommands.constEnd(); ++umcit )
 	{
-		umc = umcit.data();
+		umc = umcit.value();
 		xml.StartNewChild( XML_USERMENUCOMMAND );
-		xml.NewStringChild( XML_USERMENUCOMMAND_NAME, umc->m_sName.ascii() );
-		xml.NewStringChild( XML_USERMENUCOMMAND_ACT, umc->m_sCommand.ascii() );
+		xml.NewStringChild( XML_USERMENUCOMMAND_NAME, umc->m_sName.toAscii().constData() );
+		xml.NewStringChild( XML_USERMENUCOMMAND_ACT, umc->m_sCommand.toAscii().constData() );
 		xml.NewNumericChild( XML_USERMENUCOMMAND_CONTEXT, umc->m_nContext );
 		xml.NewNumericChild( XML_USERMENUCOMMAND_TYPE, umc->m_nType );
-		xml.NewStringChild( XML_USERMENUCOMMAND_HUBIP, umc->m_sHubIP.ascii() );
-		xml.NewNumericChild( XML_USERMENUCOMMAND_POSITION, umcit.key() ); /* ignored, the position in the file is used */
+		xml.NewStringChild( XML_USERMENUCOMMAND_HUBIP, umc->m_sHubIP.toAscii().constData() );
+		xml.NewNumericChild( XML_USERMENUCOMMAND_POSITION, umcit.key() ); /* ignored, position in file is used */
 		xml.Parent();
 	}
 	
@@ -1176,19 +1182,18 @@ int DCConfig::SaveDCGui()
 	xml.NewNumericChild( XML_SHELL_COMMAND_TIMEOUT, m_nShellCommandTimeout );
 	
 	// user chat commands
-	QPtrListIterator<DC_UserChatCommand> uccit(m_lUserChatCommands);
-	for ( ; uccit.current(); ++uccit )
+	for ( QList<DC_UserChatCommand*>::const_iterator uccit = m_lUserChatCommands.constBegin(); uccit != m_lUserChatCommands.constEnd(); ++uccit )
 	{
 		xml.StartNewChild( XML_USERCHATCOMMAND );
-		xml.NewStringChild( XML_USERCHATCOMMAND_NAME, uccit.current()->m_sName.ascii() );
-		xml.NewStringChild( XML_USERCHATCOMMAND_ACT, uccit.current()->m_sCommand.ascii() );
+		xml.NewStringChild( XML_USERCHATCOMMAND_NAME, (*uccit)->m_sName.toAscii().constData() );
+		xml.NewStringChild( XML_USERCHATCOMMAND_ACT, (*uccit)->m_sCommand.toAscii().constData() );
 		xml.Parent();
 	}
 	
 	// icon themes
-	xml.NewStringChild( XML_APP_ICON_THEME, GetApplicationIconTheme().ascii() );
-	xml.NewStringChild( XML_EMOTICON_THEME, GetEmoticonTheme().ascii() );
-	xml.NewStringChild( XML_USER_ICON_THEME, GetUserListIconTheme().ascii() );
+	xml.NewStringChild( XML_APP_ICON_THEME, GetApplicationIconTheme().toAscii().constData() );
+	xml.NewStringChild( XML_EMOTICON_THEME, GetEmoticonTheme().toAscii().constData() );
+	xml.NewStringChild( XML_USER_ICON_THEME, GetUserListIconTheme().toAscii().constData() );
 	
 	// filelist naming scheme version
 	xml.NewNumericChild( XML_FILELIST_NAMING_SCHEME, GetFilelistNamingSchemeVersion() );
@@ -1196,18 +1201,16 @@ int DCConfig::SaveDCGui()
 	// auto responses
 	xml.NewBoolChild( XML_AUTORESPONSE_ENABLED, GetAutoResponderEnabled() );
 	xml.NewBoolChild( XML_AUTORESPONSE_ENABLED_PM, GetAutoResponderEnabledForPM() );
-	QPtrListIterator<DC_AutoResponseObject> aroit(m_lAutoResponses);
-	for ( ; aroit.current(); ++aroit )
+	for ( QList<DC_AutoResponseObject*>::const_iterator arit = m_lAutoResponses.constBegin(); arit != m_lAutoResponses.constEnd(); ++arit )
 	{
-		DC_AutoResponseObject * aro = aroit.current();
 		xml.StartNewChild( XML_AUTORESPONSE );
-		xml.NewStringChild( XML_AUTORESPONSE_TRIGGER, aro->m_sTrigger.ascii() );
-		xml.NewStringChild( XML_AUTORESPONSE_RESPONSE, aro->m_sResponse.ascii() );
-		xml.NewBoolChild( XML_AUTORESPONSE_CASE_SENS, aro->m_bCaseSensitive );
+		xml.NewStringChild( XML_AUTORESPONSE_TRIGGER, (*arit)->m_sTrigger.toAscii().constData() );
+		xml.NewStringChild( XML_AUTORESPONSE_RESPONSE, (*arit)->m_sResponse.toAscii().constData() );
+		xml.NewBoolChild( XML_AUTORESPONSE_CASE_SENS, (*arit)->m_bCaseSensitive );
 		xml.Parent();
 	}
 	xml.NewNumericChild( XML_AUTORESPONSE_DELAY, GetAutoResponseDelay() );
-	xml.NewStringChild( XML_AUTORESPONSE_IGNORES, GetAutoResponseIgnores().ascii() );
+	xml.NewStringChild( XML_AUTORESPONSE_IGNORES, GetAutoResponseIgnores().toAscii().constData() );
 	
 	// save map settings
 	SettingMap::Iterator it;
@@ -1216,12 +1219,12 @@ int DCConfig::SaveDCGui()
 	for(it=m_guiSettingMap.begin();it!=m_guiSettingMap.end();++it)
 	{
 		xml.StartNewChild("MAP");
-		xml.NewStringChild( "KEY", it.key().ascii() );
-		for(it1=it.data()->begin();it1!=it.data()->end();++it1)
+		xml.NewStringChild( "KEY", it.key().toAscii().constData() );
+		for(it1=it.value()->begin();it1!=it.value()->end();++it1)
 		{
 			xml.StartNewChild( XML_ENTRY );
-			xml.NewStringChild( "KEY", it1.key().ascii() );
-			xml.NewStringChild( "NAME", it1.data().ascii() );
+			xml.NewStringChild( "KEY", it1.key().toAscii().constData() );
+			xml.NewStringChild( "NAME", it1.value().toAscii().constData() );
 			xml.Parent();
 		}
 		xml.Parent();
@@ -1290,7 +1293,7 @@ int DCConfig::LoadDCHubFilter( QMap<QString, DC_HubFilterObject*> * map )
 							}
 							else
 							{
-								(*map)[HubFilterObject->m_sFilterName] = HubFilterObject;
+								map->insert( HubFilterObject->m_sFilterName, HubFilterObject );
 							}
 						}
 					}
@@ -1316,11 +1319,11 @@ int DCConfig::SaveDCHubFilter( QMap<QString, DC_HubFilterObject*> * map )
 
 	for ( QMap<QString, DC_HubFilterObject*>::const_iterator it = map->constBegin(); it != map->constEnd(); ++it )
 	{
-		HubFilterObject = it.data();
+		HubFilterObject = it.value();
 		
 		xml.StartNewChild( XML_FILTER );
-		xml.NewStringChild( XML_FILTERNAME, HubFilterObject->m_sFilterName.ascii() );
-		xml.NewStringChild( XML_CONTAINS, HubFilterObject->m_sContains.ascii() );
+		xml.NewStringChild( XML_FILTERNAME, HubFilterObject->m_sFilterName.toAscii().constData() );
+		xml.NewStringChild( XML_CONTAINS, HubFilterObject->m_sContains.toAscii().constData() );
 		xml.NewNumericChild( XML_USER, HubFilterObject->m_nUser );
 		xml.NewBoolChild( XML_NAME, HubFilterObject->m_bName );
 		xml.NewBoolChild( XML_SERVER, HubFilterObject->m_bServer );
@@ -1412,14 +1415,14 @@ int DCConfig::SaveDCFriendList( FriendMap * map )
 
 	for ( FriendMap::const_iterator it = map->constBegin(); it != map->constEnd(); ++it )
 	{
-		FriendObject = it.data();
+		FriendObject = it.value();
 		
 		xml.StartNewChild( XML_FRIEND );
-		xml.NewStringChild( XML_NAME, FriendObject->m_sName.ascii() );
-		xml.NewStringChild( XML_DESCRIPTION, FriendObject->m_sDescription.ascii() );
-		xml.NewStringChild( XML_HUBNAME, FriendObject->m_sHubName.ascii() );
-		xml.NewStringChild( XML_HUBHOST, FriendObject->m_sHubHost.ascii() );
-		xml.NewStringChild( XML_IMAGE, FriendObject->m_sImageFileName.ascii() );
+		xml.NewStringChild( XML_NAME, FriendObject->m_sName.toAscii().constData() );
+		xml.NewStringChild( XML_DESCRIPTION, FriendObject->m_sDescription.toAscii().constData() );
+		xml.NewStringChild( XML_HUBNAME, FriendObject->m_sHubName.toAscii().constData() );
+		xml.NewStringChild( XML_HUBHOST, FriendObject->m_sHubHost.toAscii().constData() );
+		xml.NewStringChild( XML_IMAGE, FriendObject->m_sImageFileName.toAscii().constData() );
 		xml.NewBoolChild( XML_SEND_IMAGE, FriendObject->m_bSendImage );
 		xml.NewBoolChild( XML_AUTO_SEC, FriendObject->m_bAutoSec );
 		xml.NewBoolChild( XML_PERM_SLOT, FriendObject->m_bPermSlot );
@@ -1529,11 +1532,13 @@ QPixmap * DCConfig::GetUserIcon( CMessageMyInfo * myinfo )
 	if ( m_UserIconCache[x][y] == 0 )
 	{
 		m_UserIconCache[x][y] = new QPixmap(
-			m_UserIcons.copy(
-				x * USERLIST_ICON_SIZE,
-				y * USERLIST_ICON_SIZE,
-				USERLIST_ICON_SIZE,
-				USERLIST_ICON_SIZE
+			QPixmap::fromImage(
+				m_UserIcons.copy(
+					x * USERLIST_ICON_SIZE,
+					y * USERLIST_ICON_SIZE,
+					USERLIST_ICON_SIZE,
+					USERLIST_ICON_SIZE
+				)
 			)
 		);
 	}
@@ -1724,20 +1729,12 @@ void DCConfig::PlaySound( QString file )
 	}
 	else
 	{
-		if ( pProcess->isRunning() )
+		if ( pProcess->state() != QProcess::NotRunning )
 		{
 			pProcess->kill();
 		}
 
-		pProcess->clearArguments();
-
-		pProcess->addArgument(sExternalPlayer);
-		pProcess->addArgument(file);
-
-		if ( !pProcess->start() )
-		{
-			printf("error start play process\n");
-		}
+		pProcess->start( sExternalPlayer + " " + file );
 	}
 }
 
@@ -1856,15 +1853,7 @@ bool DCConfig::InitMimeSourceFactory()
 	int id = 0;
 	bool res = false;
 
-	if ( m_pMimeSourceFactory )
-	{
-		QMimeSourceFactory::removeFactory(m_pMimeSourceFactory);
-		delete m_pMimeSourceFactory;
-		m_pMimeSourceFactory = 0;
-	}
-
-	QPtrList<DC_EmoticonObject> * newemotes = new QPtrList<DC_EmoticonObject>();
-	newemotes->setAutoDelete(true);
+	QList<DC_EmoticonObject*> * newemotes = new QList<DC_EmoticonObject*>();
 
 	// load emoticons
 	
@@ -1874,13 +1863,13 @@ bool DCConfig::InitMimeSourceFactory()
 	if ( !(xml.ParseFile(s+"emoticons.xml") && xml.DocFirstChild()) )
 	{
 		// get global icons
-		s  = GetValknutDataPath().ascii();
+		s  = GetValknutDataPath().toAscii().constData();
 		s += DIRSEPARATOR;
 		s += "icons";
 		s += DIRSEPARATOR;
 		s += "emot";
 		s += DIRSEPARATOR;
-		s += GetEmoticonTheme().ascii();
+		s += GetEmoticonTheme().toAscii().constData();
 		s += DIRSEPARATOR;
 
 		if ( !(xml.ParseFile(s+"emoticons.xml") && xml.DocFirstChild()) )
@@ -1927,7 +1916,7 @@ bool DCConfig::InitMimeSourceFactory()
 							++id;
 							newemotes->append(EmoticonObject);
 
-//							printf("%s\n",EmoticonObject->m_Text.ascii());
+//							printf("%s\n",EmoticonObject->m_Text.toAscii().constData());
 					}
 					else
 					{
@@ -1941,54 +1930,46 @@ bool DCConfig::InitMimeSourceFactory()
 	}
 	while ( xml.NextNode() );
 	
-	m_EmoticonImage.reset();
 	if ( m_EmoticonImage.load((s+imagename).Data()) )
 	{
 		res = true;
 	}
 	else
 	{
+		for ( QList<DC_EmoticonObject*>::const_iterator it = newemotes->constBegin(); it != newemotes->constEnd(); ++it )
+		{
+			delete *it;
+		}
 		newemotes->clear();
 		delete newemotes;
-		newemotes = 0;
 	}
 
-	if ( newemotes )
+	if ( res )
 	{
-		if ( newemotes->count() > 0 )
+		if ( m_pEmoticons )
 		{
-			m_pMimeSourceFactory = new QMimeSourceFactory();
-
-			for ( EmoticonObject = newemotes->first(); EmoticonObject; EmoticonObject = newemotes->next() )
-			{
-				m_pMimeSourceFactory->setImage( "emoticon" + QString::number(EmoticonObject->m_nID), 
-					m_EmoticonImage.copy(EmoticonObject->left,EmoticonObject->top,EmoticonObject->right-EmoticonObject->left,EmoticonObject->bottom-EmoticonObject->top) );
-			}
-
-			QMimeSourceFactory::addFactory(m_pMimeSourceFactory);
-
-			res = true;
-			
-			if ( m_pEmoticons )
-			{
-				m_pEmoticons->clear();
-				delete m_pEmoticons;
-			}
-			
+			QList<DC_EmoticonObject*> * oldemotes = m_pEmoticons;
 			m_pEmoticons = newemotes;
+			
+			for ( QList<DC_EmoticonObject*>::const_iterator it = oldemotes->constBegin(); it != oldemotes->constEnd(); ++it )
+			{
+				delete *it;
+			}
+			oldemotes->clear();
+			delete oldemotes;
 		}
 		else
 		{
-			newemotes->clear();
-			delete newemotes;
+			m_pEmoticons = newemotes;
 		}
+		emit emoticonThemeChanged();
 	}
 
 	return res;
 }
 
 /** */
-QPtrList<DC_EmoticonObject> * DCConfig::EmoticonList()
+QList<DC_EmoticonObject*> * DCConfig::EmoticonList()
 {
 	return m_pEmoticons;
 }
@@ -1999,6 +1980,34 @@ QImage & DCConfig::GetEmoticonImage()
 	return m_EmoticonImage;
 }
 
+/** */
+bool DCConfig::AddEmoticons( QTextDocument * document )
+{
+	bool res = false;
+	DC_EmoticonObject * EmoticonObject = 0;
+	
+	if ( m_pEmoticons )
+	{
+		for ( QList<DC_EmoticonObject*>::const_iterator it = m_pEmoticons->constBegin(); it != m_pEmoticons->constEnd(); ++it )
+		{
+			EmoticonObject = *it;
+			document->addResource(
+				QTextDocument::ImageResource,
+				QUrl(GetEmoticonTheme() + QString("/emoticon") + QString().setNum(EmoticonObject->m_nID)),
+				m_EmoticonImage.copy(
+					EmoticonObject->left,
+					EmoticonObject->top,
+					EmoticonObject->right-EmoticonObject->left,
+					EmoticonObject->bottom-EmoticonObject->top
+				)
+			);
+		}
+		res = true;
+	}
+	
+	return res;
+}
+
 /** */
 bool DCConfig::GetMap( QString name, StringMap *& map )
 {
@@ -2218,7 +2227,7 @@ bool DCConfig::CheckLogChatNickNameFilter( QString nick )
 		return true;
 	}
 
-	if ( m_LogChatNickNameFilter.contains(nick) > 0 )
+	if ( m_LogChatNickNameFilter.contains(nick) )
 	{
 		if ( GetLogChatOption(elcoNICKNAMEFILTER_ENABLELOGGING) )
 		{
@@ -2385,24 +2394,24 @@ void DCConfig::SetClientColumnWidth( eClientColumn e, int i )
 
 /** */
 void DCConfig::SetUserMenuCommands( QMap<int, DC_UserMenuCommand*> * list )
-{	
+{
 	if ( !list )
 	{
 		return;
 	}
 	
 	/* clear map first delete objects second */
-	QValueList<DC_UserMenuCommand*> tmp = m_UserMenuCommands.values();
+	QList<DC_UserMenuCommand*> tmp = m_UserMenuCommands.values();
 	m_UserMenuCommands.clear();
 	
-	for ( QValueList<DC_UserMenuCommand*>::const_iterator it = tmp.constBegin(); it != tmp.constEnd(); ++it )
+	for ( QList<DC_UserMenuCommand*>::const_iterator it = tmp.constBegin(); it != tmp.constEnd(); ++it )
 	{
 		delete *it;
 	}
 	
 	for ( QMap<int, DC_UserMenuCommand*>::const_iterator it = list->constBegin(); it != list->constEnd(); ++it )
 	{
-		m_UserMenuCommands.insert( it.key(), it.data() );
+		m_UserMenuCommands.insert( it.key(), it.value() );
 	}
 	
 	list->clear();
@@ -2419,7 +2428,7 @@ long DCConfig::GetUserMenuCommands( QMap<int, DC_UserMenuCommand*> * list )
 	for ( QMap<int, DC_UserMenuCommand*>::const_iterator it = m_UserMenuCommands.constBegin(); it != m_UserMenuCommands.constEnd(); ++it )
 	{
 		/* default copy constructor does member by member copy */
-		list->insert( it.key(), new DC_UserMenuCommand( *(it.data()) ) );
+		list->insert( it.key(), new DC_UserMenuCommand( *(it.value()) ) );
 	}
 	
 	return list->size();
@@ -2428,79 +2437,79 @@ long DCConfig::GetUserMenuCommands( QMap<int, DC_UserMenuCommand*> * list )
 /** */
 long DCConfig::GetUserMenuCommandsDirect( QMap<int, DC_UserMenuCommand*> * list )
 {
-	if (!list)
+	if ( !list )
 	{
 		return 0;
 	}
 	
 	for ( QMap<int, DC_UserMenuCommand*>::const_iterator it = m_UserMenuCommands.constBegin(); it != m_UserMenuCommands.constEnd(); ++it )
 	{
-		list->insert( it.key(), it.data() );
+		list->insert( it.key(), it.value() );
 	}
 	
 	return list->size();
 }
 
 /** */
-void DCConfig::SetUserChatCommands( QPtrList<DC_UserChatCommand> * list )
+void DCConfig::SetUserChatCommands( QList<DC_UserChatCommand*> * list )
 {
 	if ( !list )
 	{
 		return;
 	}
-	
+
+	QList<DC_UserChatCommand*> tmp = m_lUserChatCommands;
 	m_lUserChatCommands.clear();
 	
-	for ( DC_UserChatCommand * ucc = list->first(); ucc; ucc = list->next() )
+	for ( QList<DC_UserChatCommand*>::const_iterator it = tmp.constBegin(); it != tmp.constEnd(); ++it )
 	{
-		m_lUserChatCommands.append( ucc );
+		delete *it;
+	}
+
+	for ( QList<DC_UserChatCommand*>::const_iterator it = list->constBegin(); it != list->constEnd(); ++it )
+	{
+		m_lUserChatCommands.append( *it );
 	}
 	
-	list->setAutoDelete(false);
 	list->clear();
 }
 
 /** */
-long DCConfig::GetUserChatCommands( QPtrList<DC_UserChatCommand> * list )
+long DCConfig::GetUserChatCommands( QList<DC_UserChatCommand*> * list )
 {
 	if ( !list )
 	{
 		return 0;
 	}
 	
-	QPtrListIterator<DC_UserChatCommand> uccit(m_lUserChatCommands);
-	for ( ; uccit.current(); ++uccit )
+	for ( QList<DC_UserChatCommand*>::const_iterator it = m_lUserChatCommands.constBegin(); it != m_lUserChatCommands.constEnd(); ++it )
 	{
 		/* default copy constructor does member by member copy */
-		list->append( new DC_UserChatCommand( *(uccit.current()) ) );
+		list->append( new DC_UserChatCommand( *(*it) ) );
 	}
 	
-	return list->count();
+	return list->size();
 }
 
 /** */
 QString DCConfig::ReplaceUserChatCommands( QString text )
 {
 	bool addedSpace = false;
-	
 	if ( text.right(1) != " " )
 	{
 		text += ' ';
 		addedSpace = true;
 	}
 	
-	QPtrListIterator<DC_UserChatCommand> uccit( m_lUserChatCommands );
-	DC_UserChatCommand * ucc;
-	while ( (ucc = uccit.current()) != 0 )
+	for ( QList<DC_UserChatCommand*>::const_iterator it = m_lUserChatCommands.constBegin(); it != m_lUserChatCommands.constEnd(); ++it )
 	{
+		DC_UserChatCommand * ucc = *it;
 		if ( text.startsWith(ucc->m_sName + " ") )
 		{
 			text.replace( 0, ucc->m_sName.length(), ucc->m_sCommand );
 		}
-		
-		++uccit;
 	}
-	
+		
 	if ( addedSpace )
 	{
 		text = text.left( text.length() - 1 );
@@ -2510,46 +2519,52 @@ QString DCConfig::ReplaceUserChatCommands( QString text )
 }
 
 /** */
-void DCConfig::SetAutoResponses( QPtrList<DC_AutoResponseObject> * list )
+void DCConfig::SetAutoResponses( QList<DC_AutoResponseObject*> * list )
 {
 	if ( !list )
 	{
 		return;
 	}
 	
+	QList<DC_AutoResponseObject*> tmp = m_lAutoResponses;
 	m_lAutoResponses.clear();
 	
-	for ( DC_AutoResponseObject * aro = list->first(); aro; aro = list->next() )
+	for ( QList<DC_AutoResponseObject*>::const_iterator it = tmp.constBegin(); it != tmp.constEnd(); ++it )
 	{
-		m_lAutoResponses.append( aro );
+		delete *it;
+	}
+	
+	for ( QList<DC_AutoResponseObject*>::const_iterator it = list->constBegin(); it != list->constEnd(); ++it )
+	{
+		m_lAutoResponses.append( *it );
 	}
 	
-	list->setAutoDelete(false);
 	list->clear();
 }
 
 /** */
-long DCConfig::GetAutoResponses( QPtrList<DC_AutoResponseObject> * list )
+long DCConfig::GetAutoResponses( QList<DC_AutoResponseObject*> * list )
 {
 	if ( !list )
 	{
 		return 0;
 	}
 	
-	QPtrListIterator<DC_AutoResponseObject> aroit(m_lAutoResponses);
-	for ( ; aroit.current(); ++aroit )
+	for ( QList<DC_AutoResponseObject*>::const_iterator it = m_lAutoResponses.constBegin(); it != m_lAutoResponses.constEnd(); ++it )
 	{
 		/* default copy constructor does member by member copy */
-		list->append( new DC_AutoResponseObject( *(aroit.current()) ) );
+		list->append( new DC_AutoResponseObject( *(*it) ) );
 	}
 	
-	return list->count();
+	return list->size();
 }
 
 void DCConfig::RenameStoredFilelists()
 {
 	QDir dir;
 	QStringList listing;
+	QByteArray buffer;
+	buffer.resize(2048);
 	
 	printf("Renaming stored filelists ...\n");
 	
@@ -2568,17 +2583,19 @@ void DCConfig::RenameStoredFilelists()
 		else
 		{
 			QFile file;
-			file.setName( g_pConfig->GetFileListPath().Data() + filename );
-			if (file.open( IO_ReadOnly ))
+			file.setFileName( QString(g_pConfig->GetFileListPath().Data()) + filename );
+			if (file.open( QIODevice::ReadOnly ))
 			{
 				QString line, nick, hubhost;
-				file.readLine( line, (Q_ULONG) 2048 );
+				file.readLine( buffer.data(), buffer.size() );
+				line = buffer.constData();
 				
 				if (line == "---HEADER START---\n")
 				{
 					while (line != "---HEADER END---\n")
 					{
-						file.readLine( line, (Q_ULONG) 2048 );
+						file.readLine( buffer.data(), buffer.size() );
+						line = buffer.constData();
 						
 						if (line.left(5) == "NICK=")
 						{
@@ -2594,7 +2611,7 @@ void DCConfig::RenameStoredFilelists()
 				}
 				else
 				{
-					printf("%s is not a filelist from valknut <= 0.3.13\n", filename.ascii());
+					printf("%s is not a filelist from valknut <= 0.3.13\n", filename.toAscii().constData());
 					file.close();
 					continue;
 				}
@@ -2614,7 +2631,7 @@ void DCConfig::RenameStoredFilelists()
 					continue;
 				}
 				
-				printf("Renaming %s to %s ...", filename.ascii(), newname.ascii());
+				printf("Renaming %s to %s ...", filename.toAscii().constData(), newname.toAscii().constData());
 				if ( dir.rename(filename, newname) )
 				{
 					printf(" OK\n");
@@ -2626,7 +2643,7 @@ void DCConfig::RenameStoredFilelists()
 			}
 			else
 			{
-				printf("Couldn't open %s!\n", filename.ascii());
+				printf("Couldn't open %s!\n", filename.toAscii().constData());
 			}
 		}
 	}
@@ -2656,7 +2673,6 @@ void DCConfig::SetDefaultDownloadMode( unsigned int n, bool emitsig )
 	if ( n != iDefaultDownloadMode )
 	{
 		iDefaultDownloadMode = n;
-		
 		if ( emitsig )
 		{
 			emit defaultDownloadModeChanged();
@@ -2667,27 +2683,14 @@ void DCConfig::SetDefaultDownloadMode( unsigned int n, bool emitsig )
 /** */
 void DCConfig::OpenURL( QString url )
 {
-	QString browser;
 	if ( m_bCustomBrowser && !sBrowser.isEmpty() )
 	{
-		browser = sBrowser;
+		QProcess::startDetached( sBrowser + " " + url );
 	}
 	else
 	{
-#if defined(Q_OS_MACX)
-		browser = "open";
-#elif defined(Q_WS_WIN)
-		browser = "start";
-#else
-		browser = "xdg-open";
-#endif
+		QDesktopServices::openUrl( url );
 	}
-
-	QProcess * pr = new QProcess(this);
-	pr->addArgument(browser);
-	pr->addArgument(url);
-	pr->launch(QString());
-	delete pr;
 }
 
 /** */
