@@ -22,20 +22,19 @@
   *@author Mathias Küster
   */
 
-#include <qlistview.h>
-#include <qheader.h>
 #include <qimage.h>
 #include <qobject.h>
-#include <qmap.h>
-#include <qstringlist.h>
-#include <qptrlist.h>
-
-#include <dclib/cconfig.h>
+#include <QByteArray>
+#include <QMap>
+#include <QList>
+#include <QStringList>
 
 // the typedef in dcfriendobject.h prevents the forward
 // declaration of "class FriendMap;" from working
 #include "dcfriendobject.h"
 
+#include <dclib/cconfig.h>
+
 #define USERLIST_XPM_COLUMNS      9
 #define USERLIST_XPM_ROWS        32
 #define USERLIST_ICON_SIZE       16
@@ -138,6 +137,7 @@ typedef enum {
 
 class QPixmap;
 class QProcess;
+class QTextDocument;
 class CXml;
 class CMessageMyInfo;
 
@@ -190,7 +190,7 @@ public:
 };
 
 /** Stores a user command for custom menus */
-class DC_UserMenuCommand  {
+class DC_UserMenuCommand {
 public:
 	/** */
 	DC_UserMenuCommand() {
@@ -454,13 +454,13 @@ public:
 	QString GetBrowser() const;
 	/** */
 	void SetBrowser( QString s );
-	/** Open URL with the browser */
+	/** Open the URL using the browser settings */
 	void OpenURL( QString url );
 	/** */
 	bool GetCustomBrowser() const;
 	/** */
 	void SetCustomBrowser( bool b );
-	
+
 	/** */
 	unsigned int GetDefaultDownloadMode() const;
 	/** 0:ask, 1:single, 2:multi */
@@ -491,7 +491,7 @@ public:
 	QString GetChatColor( eChatColor type ) const;
 	/** */
 	void SetChatColor( eChatColor type, QString s );
-	/** */
+ 	/** */
 	QString GetChatBackgroundColor() const;
 	/** */
 	bool GetChatBackgroundColorEnabled() const;
@@ -519,9 +519,11 @@ public:
 	/** */
 	bool InitMimeSourceFactory();
 	/** */
-	QPtrList<DC_EmoticonObject> * EmoticonList();
+	QList<DC_EmoticonObject*> * EmoticonList();
 	/** */
 	QImage & GetEmoticonImage();
+	/** adds emoticon resources to the text document */
+	bool AddEmoticons( QTextDocument * document );
 
 	/** */
 	QString GetUserPhotoFileName() const;
@@ -534,16 +536,16 @@ public:
 	void SetValknutDataPath( QString s );
 	
 	/** Get the list of user chat commands, caller must delete them */
-	long GetUserChatCommands( QPtrList<DC_UserChatCommand> * list );
+	long GetUserChatCommands( QList<DC_UserChatCommand*> * list );
 	/** Set the list of user chat commands, transfers ownership of the objects to g_pConfig */
-	void SetUserChatCommands( QPtrList<DC_UserChatCommand> * list );
+	void SetUserChatCommands( QList<DC_UserChatCommand*> * list );
 	/**
 	 * Instead of copying the list of commands to a DCChat and then deleting them all,
 	 * send the text to g_pConfig and get the text with the commands substituted.
 	 */
 	QString ReplaceUserChatCommands( QString text );
 	
-	/** Get a copy of the list of user menu commands */
+	/** Get a copy of the list of user menu commands, caller must delete them */
 	long GetUserMenuCommands( QMap<int, DC_UserMenuCommand*> * list );
 	/** Get pointers to the actual user menu commands */
 	long GetUserMenuCommandsDirect( QMap<int, DC_UserMenuCommand*> * list );
@@ -588,9 +590,9 @@ public:
 	/** Enables/Disables the auto responder for main chat */
 	void SetAutoResponderEnabled( bool b );
 	/** Gets the list of auto response triggers and responses, the caller must delete them */
-	long GetAutoResponses( QPtrList<DC_AutoResponseObject> * list );
+	long GetAutoResponses( QList<DC_AutoResponseObject*> * list );
 	/** Sets the list of auto response triggers and responses, transfers ownership of the objects to g_pConfig */
-	void SetAutoResponses( QPtrList<DC_AutoResponseObject> * list );
+	void SetAutoResponses( QList<DC_AutoResponseObject*> * list );
 	/** Gets the minimum time between auto-responses */
 	long GetAutoResponseDelay() const;
 	/** Sets the minimum time between auto-responses */
@@ -604,13 +606,28 @@ public:
 	/** Enables/Disabled the auto responder for private messages */
 	void SetAutoResponderEnabledForPM( bool b );
 	
-	/** Gets filelist naming scheme version */
+	/** Gets filelist naming scheme version
+	  * This is set internally so that the filelists are renamed
+	  * once only when upgrading to 0.3.10.
+	  * once only when upgrading to 0.3.17.
+	  *
+	  * 0 = 0.3.8 or earlier integers
+	  * 1 = 0.3.9 base64 encoded
+	  * 2 = 0.3.10 human readable nick-hubhost_port.filelist
+	  * 3 = setting was abused for searchindex changes
+	  * 4 = 0.3.17 human readable nick@hubhost_port.filelist
+	  */
 	int GetFilelistNamingSchemeVersion() const;
 	/** Sets filelist naming scheme version */
 	void SetFilelistNamingSchemeVersion( int i );
 	/** Renames the filelists */
 	void RenameStoredFilelists();
 	
+	/** Gets QMainWindow saved state */
+	QByteArray GetQMainWindowState() const;
+	/** Sets QMainWindow state */
+	void SetQMainWindowState( QByteArray b );
+	
 	/** Get show folders in right pane of filelist browser*/
 	bool GetFoldersInRightPane() const;
 	/** Set show folders in right pane of filelist browser */
@@ -674,13 +691,15 @@ public:
 	eMagnetAction GetMagnetAction( eMagnetSource = eChatMagnet ) const;
 	/** Set default magnet action */
 	void SetMagnetAction( eMagnetAction act, eMagnetSource = eChatMagnet );
-
+	
 signals:
+	/** emitted when the emoticon theme has changed */
+	virtual void emoticonThemeChanged();
 	/** emitted after chat background color settings have changed */
 	virtual void chatBackgroundColorChanged();
 	/** emitted after the default download mode has been changed not from the main menu */
 	virtual void defaultDownloadModeChanged();
-
+	
 private:
 	/** */
 	QString m_sValknutDataPath;
@@ -706,8 +725,8 @@ private:
 	eUnit m_eUnit;
 	/** */
 	QString sTheme;
-	/** Needed to avoid removing the QT4 verion's theme setting */
-	QString sThemeQT4;
+	/** Needed to avoid removing the QT3 version's theme setting */
+	QString sThemeQT3;
 	/** */
 	int m_nSendChat;
 	/** */
@@ -801,8 +820,6 @@ private:
 
 	/** */
 	int iChatMessageMaxLines;
-	/** */
-	QMimeSourceFactory * m_pMimeSourceFactory;
 	
 	/** */
 	int iFloodCount;
@@ -810,7 +827,7 @@ private:
 	bool m_bFloodOpKick;
 
 	/** */
-	QPtrList<DC_EmoticonObject> * m_pEmoticons;
+	QList<DC_EmoticonObject*> * m_pEmoticons;
 	/** */
 	QImage m_EmoticonImage;
 
@@ -841,7 +858,7 @@ private:
 	QString sBackgroundPixmap;
 	/** */
 	QString sBrowser;
-	/** false to use reasonable default xdg-open */
+	/** true to use sBrowser, false to use QDesktopServices */
 	bool m_bCustomBrowser;
 	/** */
 	bool bShowChatInTab;
@@ -934,9 +951,9 @@ private:
 	QString m_sUserPhotoFileName;
 	
 	/** The list of user customized chat commands */
-	QPtrList<DC_UserChatCommand> m_lUserChatCommands;
+	QList<DC_UserChatCommand*> m_lUserChatCommands;
 
-	/** The list of user customized menu commands */
+	/** The list of user customized menu commands - now in order! */
 	QMap<int, DC_UserMenuCommand*> m_UserMenuCommands;
 	
 	/** Theme for application icons */
@@ -965,7 +982,7 @@ private:
 	bool m_bAutoResponderEnabledForPM;
 	
 	/** The list of auto response triggers and responses */
-	QPtrList<DC_AutoResponseObject> m_lAutoResponses;
+	QList<DC_AutoResponseObject*> m_lAutoResponses;
 	
 	/** The time between consequtive auto-responses for a user */
 	long m_nAutoResponseDelay;
@@ -975,18 +992,12 @@ private:
 	
 	/** The filelist naming scheme version.
 	  * This is set internally so that the filelists are renamed
-	  * once only when upgrading to 0.3.17.
-	  *
-	  * 0 = 0.3.8 or earlier integers
-	  * 1 = 0.3.9 base64 encoded
-	  * 2 = 0.3.10 human readable nick-hubhost_port.filelist
-	  * 3 = setting was abused for searchindex changes
-	  * 4 = 0.3.17 human readable nick@hubhost_port.filelist
+	  * once only when upgrading to 0.3.10.
 	  */
 	int m_nFilelistNamingSchemeVersion;
 	
 	/** Saved QT4 QMainWindow state */
-	QString m_sQT4MainWindowState;
+	QByteArray m_baQMainWindowState;
 	
 	/** Show folders in right pane of filelist browser? */
 	bool m_bFoldersInRightPane;
@@ -1329,6 +1340,12 @@ inline int DCConfig::GetFilelistNamingSchemeVersion() const
 inline void DCConfig::SetFilelistNamingSchemeVersion( int i )
 { m_nFilelistNamingSchemeVersion = i; }
 /** */
+inline QByteArray DCConfig::GetQMainWindowState() const
+{ return m_baQMainWindowState; }
+/** */
+inline void DCConfig::SetQMainWindowState( QByteArray ba )
+{ m_baQMainWindowState = ba; }
+/** */
 inline bool DCConfig::GetFoldersInRightPane() const
 { return m_bFoldersInRightPane; }
 /** */
