@@ -18,19 +18,20 @@
 #include "dcfilebrowser.h"
 
 #include <qapplication.h>
-#include <qheader.h>
-#include <qlistview.h>
+#include <q3header.h>
+#include <q3listview.h>
 #include <qlabel.h>
-#include <qpopupmenu.h>
 #include <qcursor.h>
-#include <qfiledialog.h>
+#include <QFileDialog>
+#include <QUrl>
 #include <qtoolbutton.h>
 #include <qcombobox.h>
 #include <qtooltip.h>
 #include <qclipboard.h>
 #include <qmessagebox.h>
-#include <qworkspace.h>
-#include <qptrlist.h>
+#include <qstyle.h>
+#include <qmdiarea.h>
+#include <qlist.h>
 
 #include "dcmenuhandler.h"
 #include "dcconfig.h"
@@ -43,8 +44,6 @@
 #include "dcclient.h"
 #include "dcconnectionmanager.h"
 
-#include "dcguiutils.h"
-
 // for searching by TTH
 #include "dchubsearch.h"
 
@@ -58,25 +57,29 @@
 // for file type icons
 #include <dclib/cfilemanager.h>
 
+#include "dcguiutils.h"
+
 /** */
-DCFileBrowser::DCFileBrowser( QWidget* parent, const char *name, int wflags, bool allowopen )
-	: DCDialogFileBrowser( parent, name, wflags )
+DCFileBrowser::DCFileBrowser( QWidget * parent, bool allowopen )
+	: QWidget( parent )
 {
+	setupUi(this);
+	
 	m_bAllowOpen = allowopen;
 	m_bAllowDownload = true;
 	
 	// set caption
-	setCaption(tr("Filebrowser"));
-	setIcon( g_pIconLoader->GetPixmap(eiVIEW_SIDETREE) );
+	setWindowTitle(tr("Filebrowser"));
+	setWindowIcon( g_pIconLoader->GetPixmap(eiVIEW_SIDETREE) );
 	
 	// set width mode
-	ListView_DIRECTORY->setColumnWidthMode( 0, QListView::Maximum );
-	ListView_FILES->setColumnWidthMode( 0, QListView::Maximum );
-	ListView_FILES->setColumnWidthMode( 1, QListView::Maximum );
-	ListView_FILES->setColumnWidthMode( 2, QListView::Maximum );
-	ListView_FILES->setColumnWidthMode( 3, QListView::Maximum );
-	ListView_FILES->setColumnWidthMode( 4, QListView::Maximum );
-	
+	ListView_DIRECTORY->setColumnWidthMode( 0, Q3ListView::Maximum );
+	ListView_FILES->setColumnWidthMode( 0, Q3ListView::Maximum );
+	ListView_FILES->setColumnWidthMode( 1, Q3ListView::Maximum );
+	ListView_FILES->setColumnWidthMode( 2, Q3ListView::Maximum );
+	ListView_FILES->setColumnWidthMode( 3, Q3ListView::Maximum );
+	ListView_FILES->setColumnWidthMode( 4, Q3ListView::Maximum );
+
 	// set right alignment on size columns
 	ListView_DIRECTORY->setColumnAlignment( 1, Qt::AlignRight );
 	ListView_FILES->setColumnAlignment( 1, Qt::AlignRight );
@@ -93,6 +96,10 @@ DCFileBrowser::DCFileBrowser( QWidget* parent, const char *name, int wflags, boo
 
 	m_pDecomp = 0;
 
+	ToolButton_OPEN->setIcon( style()->standardIcon(QStyle::SP_DialogOpenButton,0,ToolButton_OPEN) );
+	ToolButton_SAVE->setIcon( style()->standardIcon(QStyle::SP_DialogSaveButton,0,ToolButton_SAVE) );
+	ToolButton_GOTO_USER->setIcon( g_pIconLoader->GetPixmap(eiUSERS) );
+	
 	InitDocument();
 }
 
@@ -122,11 +129,11 @@ DCFileBrowser::~DCFileBrowser()
 void DCFileBrowser::InitDocument()
 {
 	connect( ListView_DIRECTORY, SIGNAL(selectionChanged()), this, SLOT(slotCurrentChangedDirectory()) );
-	connect( ListView_DIRECTORY, SIGNAL(expanded(QListViewItem*)), this, SLOT(slotExpandedDirectory(QListViewItem*)) );
-	connect( ListView_DIRECTORY, SIGNAL(collapsed(QListViewItem*)), this, SLOT(slotCollapsedDirectory(QListViewItem*)) );
-	connect( ListView_DIRECTORY, SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedDirectory(QListViewItem*, const QPoint &, int )) );
-	connect( ListView_FILES, SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedFiles(QListViewItem*, const QPoint &, int )) );
-	connect( ListView_FILES, SIGNAL(doubleClicked( QListViewItem *, const QPoint &, int)), this, SLOT(slotItemDoubleClicked( QListViewItem *, const QPoint &, int )) );
+	connect( ListView_DIRECTORY, SIGNAL(expanded(Q3ListViewItem*)), this, SLOT(slotExpandedDirectory(Q3ListViewItem*)) );
+	connect( ListView_DIRECTORY, SIGNAL(collapsed(Q3ListViewItem*)), this, SLOT(slotCollapsedDirectory(Q3ListViewItem*)) );
+	connect( ListView_DIRECTORY, SIGNAL(contextMenuRequested( Q3ListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedDirectory(Q3ListViewItem*, const QPoint &, int )) );
+	connect( ListView_FILES, SIGNAL(contextMenuRequested( Q3ListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedFiles(Q3ListViewItem*, const QPoint &, int )) );
+	connect( ListView_FILES, SIGNAL(doubleClicked( Q3ListViewItem *, const QPoint &, int)), this, SLOT(slotItemDoubleClicked( Q3ListViewItem *, const QPoint &, int )) );
 
  	connect( ToolButton_OPEN, SIGNAL(clicked()), this, SLOT(slotFileOpen()) );
  	connect( ToolButton_SAVE, SIGNAL(clicked()), this, SLOT(slotFileSave()) );
@@ -136,34 +143,10 @@ void DCFileBrowser::InitDocument()
 	ToolButton_GOTO_USER->setEnabled(false);
 }
 
-void DCFileBrowser::customEvent( QCustomEvent * event )
+/** */
+void DCFileBrowser::customEvent( QEvent * event )
 {
-	if ( event->type() == EVENT_FILELIST_DECOMPRESSED )
-	{
-		if ( ((DC_FilelistDecompressedEvent*)event)->ok )
-		{
-			if ( m_pDecomp->recompressed != 0 )
-			{
-				m_ListData.SetSize(0);
-				m_ListData.Append( m_pDecomp->recompressed->Data(), m_pDecomp->recompressed->Size() );
-				delete m_pDecomp->recompressed;
-			}
-			ContinueInitTree( m_pDecomp->getData() );
-		}
-		else
-		{
-			QMessageBox::critical(
-				this,
-				tr("Failed to load filelist"),
-				tr("Unable to load") + " " + m_sFileName
-			);
-			setCaption(tr("Filebrowser"));
-			EnableGuiElements();
-		}
-		delete m_pDecomp;
-		m_pDecomp = 0;
-	}
-	else if ( event->type() == EVENT_OPEN_FILELIST_FOLDER )
+	if ( event->type() == EVENT_OPEN_FILELIST_FOLDER )
 	{
 		DCFileBrowserListItem * item = ((DC_OpenFilelistFolderEvent*)event)->m_pItem;
 		
@@ -171,22 +154,59 @@ void DCFileBrowser::customEvent( QCustomEvent * event )
 		ListView_DIRECTORY->ensureItemVisible( item );
 		ListView_DIRECTORY->setSelected( item, true );
 		ListView_DIRECTORY->setCurrentItem( item );
+		
+		event->accept();
 	}
 	else
 	{
-		QWidget::customEvent( event );
+		event->ignore();
 	}
 }
 
+/** */
+void DCFileBrowser::slotThreadFinishedLoading( bool ok )
+{
+	if ( ok )
+	{
+		if ( m_pDecomp->recompressed != 0 )
+		{
+			m_ListData.SetSize(0);
+			m_ListData.Append( m_pDecomp->recompressed->Data(), m_pDecomp->recompressed->Size() );
+			delete m_pDecomp->recompressed;
+		}
+		ContinueInitTree( m_pDecomp->getData() ); // which will delete the data when it finishes
+	}
+	else
+	{
+		QMessageBox::critical(
+			this,
+			tr("Failed to load filelist"),
+			tr("Unable to load") + " " + m_sFileName
+		);
+		setWindowTitle(tr("Filebrowser"));
+		EnableGuiElements();
+	}
+	
+	disconnect( m_pDecomp, SIGNAL(finished(bool)), this, SLOT(slotThreadFinishedLoading(bool)) );
+	delete m_pDecomp;
+	m_pDecomp = 0;
+}
+
 /** */
 void DCFileBrowser::ClearView()
 {
 	ListView_FILES->clear();
 
-	QListViewItemIterator it( ListView_DIRECTORY );
+	Q3ListViewItemIterator it( ListView_DIRECTORY );
 
 	for ( ; it.current(); it++ )
 	{
+		// no longer have auto-delete with QT4 QList
+		for ( int i = 0; i < ((DCFileBrowserListItem*)it.current())->m_pFileList.size(); i++ )
+		{
+			delete ((DCFileBrowserListItem*)it.current())->m_pFileList.at(i);
+		}
+		
 		((DCFileBrowserListItem*)it.current())->m_pFileList.clear();
 	}
 
@@ -207,16 +227,16 @@ void DCFileBrowser::slotFileOpen()
 	}
 	
 	QString s = QFileDialog::getOpenFileName(
+		this,
+		tr("Choose a file"),
 		file,
 		tr("Modern XML Filelists") + " (*.xml.bz2);;" +
 		tr("Modern XML Filelists (uncompressed)") + " (*.xml);;" +
 		tr("Old BZ2 Text Filelists") + " (*.bz2);;" +
 		tr("Very old HE3 Text Filelists") + " (*.DcLst);;" +
 		tr("Old Valknut Filelists") + " (*.filelist);;" +
-		tr("All files") + " (*)",
-		this,
-		tr("open filelist dialog"),
-		tr("Choose a file") );
+		tr("All files") + " (*)"
+	);
 
 	if ( !s.isEmpty() )
 	{
@@ -238,18 +258,17 @@ void DCFileBrowser::slotFileSave()
 	}
 	
 	QString s = QFileDialog::getSaveFileName(
-                    m_sFileName,
-                    QString(),
-                    this,
-                    tr("save file dialog"),
-                    tr("Choose a filename to save under") );
+		this,
+		tr("Choose a filename to save under"),
+		m_sFileName
+	);
 
 	if ( s.isEmpty() )
 	{
 		return;
 	}
 	
-	if ( m_ListData.SaveToFile(s.ascii()) == false )
+	if ( m_ListData.SaveToFile(s.toAscii().constData()) == false )
 	{
 		QMessageBox::critical(
 			this,
@@ -264,11 +283,11 @@ void DCFileBrowser::slotCurrentChangedDirectory()
 {
 	DCFileItem * FileItem;
 	QString s;
-	QPtrList<QListViewItem> selitems;
+	QList<Q3ListViewItem*> selitems;
 	ulonglong size=0;
 	int items = 0;
 	int citems = 0;
-	QListViewItem * item = 0;
+	Q3ListViewItem * item = 0;
 
 	// clear filelist
 	ListView_FILES->clear();
@@ -278,8 +297,10 @@ void DCFileBrowser::slotCurrentChangedDirectory()
 	{
 		item = selitems.first();
 
-		for ( FileItem = ((DCFileBrowserListItem*)item)->m_pFileList.first(); FileItem; FileItem = ((DCFileBrowserListItem*)item)->m_pFileList.next() )
+		for ( int i = 0; i < ((DCFileBrowserListItem*)item)->m_pFileList.size(); i++ )
 		{
+			FileItem = ((DCFileBrowserListItem*)item)->m_pFileList.at(i);
+			
 			if ( (g_pConfig->GetFoldersInRightPane() == false) && FileItem->m_bIsDir )
 			{
 				// do not show it
@@ -293,7 +314,7 @@ void DCFileBrowser::slotCurrentChangedDirectory()
 			item1->mycol   = 1;
 			item1->m_bSortTop = false;
 			item1->pDirItem = ((DCFileBrowserListItem*)item);
-			
+
 			if ( FileItem->m_bIsDir )
 			{
 				s = tr("Folder");
@@ -306,15 +327,15 @@ void DCFileBrowser::slotCurrentChangedDirectory()
 			}
 			else
 			{
-				s = CDir::Extension(FileItem->m_sName.ascii()).Data();
+				s = CDir::Extension(FileItem->m_sName.toAscii().constData()).Data();
 
 				if ( !s.isEmpty() )
 				{
 					s += " ";
 					s += tr("file");
 				}
-			
-				eFileTypes type = CFileManager::Instance()->GetFileType(FileItem->m_sName.ascii());
+				
+				eFileTypes type = CFileManager::Instance()->GetFileType(FileItem->m_sName.toAscii().constData());
 				QPixmap icon;
 				
 				switch (type)
@@ -343,7 +364,7 @@ void DCFileBrowser::slotCurrentChangedDirectory()
 				}
 				
 				item1->setPixmap( 0, icon );
-				
+
 				items++;
 			}
 
@@ -367,35 +388,34 @@ void DCFileBrowser::slotCurrentChangedDirectory()
 }
 
 /** */
-void DCFileBrowser::slotExpandedDirectory( QListViewItem * item )
+void DCFileBrowser::slotExpandedDirectory( Q3ListViewItem * item )
 {
 	if ( item->parent() != NULL )
 		item->setPixmap(0,g_pIconLoader->GetPixmap(eiFOLDER_BLUE_OPEN));
 }
 
 /** */
-void DCFileBrowser::slotCollapsedDirectory( QListViewItem * item )
+void DCFileBrowser::slotCollapsedDirectory( Q3ListViewItem * item )
 {
 	if ( item->parent() != NULL )
 		item->setPixmap(0,g_pIconLoader->GetPixmap(eiFOLDER_BLUE));
 }
 
 /** */
-void DCFileBrowser::slotRightButtonClickedDirectory( QListViewItem * item, const QPoint &, int column )
+void DCFileBrowser::slotRightButtonClickedDirectory( Q3ListViewItem * item, const QPoint &, int column )
 {
 	CreateMenu(item,true,column);
 }
 
 /** */
-void DCFileBrowser::slotRightButtonClickedFiles( QListViewItem * item, const QPoint &, int column )
+void DCFileBrowser::slotRightButtonClickedFiles( Q3ListViewItem * item, const QPoint &, int column )
 {
 	CreateMenu(item,false,column);
 }
 
 /** */
-void DCFileBrowser::AddDirectory( QListViewItem * item, QString name )
+void DCFileBrowser::AddDirectory( Q3ListViewItem * item, QString name )
 {
-	((DCFileBrowserListItem*)item)->m_pFileList.setAutoDelete(true);
 	((DCFileBrowserListItem*)item)->m_sName = name;
 
 	item->setText(0,name);
@@ -405,7 +425,7 @@ void DCFileBrowser::AddDirectory( QListViewItem * item, QString name )
 }
 
 /** */
-void DCFileBrowser::AddFile( QListViewItem * item, DCFileItem * fileitem )
+void DCFileBrowser::AddFile( Q3ListViewItem * item, DCFileItem * fileitem )
 {
 	// increment filecounter
 	if ( fileitem->m_bIsDir == false )
@@ -479,6 +499,7 @@ void DCFileBrowser::InitTree( QString nick, QString hubname, QString hubhost, QS
 		
 		DisableGuiElements();
 		m_pDecomp = new DCFilelistDecompressor( this, &m_ListData );
+		connect( m_pDecomp, SIGNAL(finished(bool)), this, SLOT(slotThreadFinishedLoading(bool)) );
 		m_pDecomp->start();
 		return;
 	}
@@ -497,7 +518,8 @@ void DCFileBrowser::InitTree( QString nick, QString hubname, QString hubhost, QS
 		}
 		
 		DisableGuiElements();
-		m_pDecomp = new DCFilelistDecompressor( this, filename.ascii() );
+		m_pDecomp = new DCFilelistDecompressor( this, filename.toAscii().constData() );
+		connect( m_pDecomp, SIGNAL(finished(bool)), this, SLOT(slotThreadFinishedLoading(bool)) );
 		m_pDecomp->start();
 		return;
 	}
@@ -514,19 +536,19 @@ void DCFileBrowser::ContinueInitTree( CString * data )
 			tr("Error opening filelist"),
 			m_sFileName + " " + tr("is likely not a valid filelist")
 		);
-		setCaption(tr("Filebrowser"));
+		setWindowTitle(tr("Filebrowser"));
 		EnableGuiElements();
 		return;
 	}
-
-	QListViewItem * item;
 	
-	ClearView();
+	Q3ListViewItem * item;
 	
 	// reset count values
 	m_nShareSize = 0;
 	m_nFileCount = 0;
 
+	ClearView();
+
 	// get folder pixmap
 	m_FolderPixmap = g_pIconLoader->GetPixmap(eiFOLDER_BLUE);
 
@@ -540,7 +562,7 @@ void DCFileBrowser::ContinueInitTree( CString * data )
 	
 	if ( (m_sNick.isEmpty()) || (m_sHubName.isEmpty()) || (m_sHubHost.isEmpty()) )
 	{
-		int i = fname.find('@');
+		int i = fname.indexOf('@');
 		if ( i != -1 )
 		{
 			m_sNick = fname.left(i);
@@ -552,7 +574,7 @@ void DCFileBrowser::ContinueInitTree( CString * data )
 			m_sHubHost.remove(".filelist");
 			
 			// replace _ with : before port number
-			int i2 = m_sHubHost.findRev('_');
+			int i2 = m_sHubHost.lastIndexOf('_');
 			if ( i2 != -1 )
 			{
 				bool ok;
@@ -584,11 +606,11 @@ void DCFileBrowser::ContinueInitTree( CString * data )
 	
 	if ( m_sHubName.isEmpty() )
 	{
-		setCaption( m_sNick + " - " + tr("Filebrowser") );
+		setWindowTitle( m_sNick + " - " + tr("Filebrowser") );
 	}
 	else
 	{
-		setCaption( m_sNick + " - " + tr("Filebrowser") + " [" + m_sHubName + "]" );
+		setWindowTitle( m_sNick + " - " + tr("Filebrowser") + " [" + m_sHubName + "]" );
 	}
 	
 	// create root item
@@ -596,13 +618,12 @@ void DCFileBrowser::ContinueInitTree( CString * data )
 	item->setText(0,tr("Root Directory"));
 	item->setPixmap(0,g_pIconLoader->GetPixmap(eiFOLDER_RED));
 	((DCFileBrowserListItem*)item)->m_pFileItem = 0;
-	((DCFileBrowserListItem*)item)->m_pFileList.setAutoDelete(true);
-	
+
 	if ( data->Left(9).Find("<?xml") != -1 )
-		InitXMLTree(item,data);
+		InitXMLTree(item,data);			
 	else
 		InitTXTTree(item,data);
-
+	
 	delete data;
 
 	// update summary
@@ -610,7 +631,7 @@ void DCFileBrowser::ContinueInitTree( CString * data )
 					DCGuiUtils::GetSizeString(m_nShareSize) + " " + tr("Total") + ")" );
 
 	// update tooltip
-	QToolTip::add( TextLabel_SUMMARY, QString().setNum(m_nShareSize) + " B" );
+	TextLabel_SUMMARY->setToolTip( QString().setNum(m_nShareSize) + " B" );
 
 	// update view
 	ListView_DIRECTORY->setUpdatesEnabled(true);
@@ -633,12 +654,13 @@ void DCFileBrowser::ContinueInitTree( CString * data )
 	{
 		JumpTo( m_sJumpTo );
 	}
+	
 }
 
 /** */
 void DCFileBrowser::DisableGuiElements()
 {
-	setCaption(tr("Processing filelist..."));
+	setWindowTitle(tr("Processing filelist..."));
 	ToolButton_OPEN->setEnabled(false);
 	ToolButton_SAVE->setEnabled(false);
 	ToolButton_GOTO_USER->setEnabled(false);
@@ -664,7 +686,7 @@ void DCFileBrowser::EnableGuiElements()
 }
 
 /** */
-void DCFileBrowser::InitTXTTree( QListViewItem * item, CString * data )
+void DCFileBrowser::InitTXTTree( Q3ListViewItem * item, CString * data )
 {
 	QString s1,sname,slength;
 	long i,j,d;
@@ -677,7 +699,7 @@ void DCFileBrowser::InitTXTTree( QListViewItem * item, CString * data )
 		s1 = data->Mid(j,i-j).Data();
 
 		depth=0;
-		while( (s1.find("\x9",depth)) != -1 )
+		while( (s1.indexOf("\x9",depth)) != -1 )
 			depth++;
 		s1.remove( "\x9" );
 
@@ -691,7 +713,7 @@ void DCFileBrowser::InitTXTTree( QListViewItem * item, CString * data )
 
 		if ( !s1.isEmpty() )
 		{
-			d = s1.findRev("|");
+			d = s1.lastIndexOf("|");
 
 			if ( d != -1 ) // parse file entry
 			{
@@ -728,7 +750,7 @@ void DCFileBrowser::InitTXTTree( QListViewItem * item, CString * data )
 	}
 	
 	// fixup the folder sizes
-	QListViewItem * cur = ListView_DIRECTORY->firstChild();
+	Q3ListViewItem * cur = ListView_DIRECTORY->firstChild();
 	while ( cur )
 	{
 		CalcDirSize( cur );
@@ -738,7 +760,7 @@ void DCFileBrowser::InitTXTTree( QListViewItem * item, CString * data )
 }
 
 /** */
-void DCFileBrowser::InitXMLTree( QListViewItem * item, CString * data )
+void DCFileBrowser::InitXMLTree( Q3ListViewItem * item, CString * data )
 {
 	CXml * xml = new CXml();
 	
@@ -762,7 +784,7 @@ void DCFileBrowser::InitXMLTree( QListViewItem * item, CString * data )
 }
 
 /** */
-void DCFileBrowser::ParseXMLTree( CXml * xml, QListViewItem * item )
+void DCFileBrowser::ParseXMLTree( CXml * xml, Q3ListViewItem * item )
 {
 	CString s;
 
@@ -781,7 +803,7 @@ void DCFileBrowser::ParseXMLTree( CXml * xml, QListViewItem * item )
 		}
 		else if ( xml->Name() == "Directory" )
 		{
-			QListViewItem * item1;
+			Q3ListViewItem * item1;
 			
 			item1 = new DCFileBrowserListItem( item );
 			((DCFileBrowserListItem*)item1)->m_pFileItem = 0;
@@ -801,10 +823,10 @@ void DCFileBrowser::ParseXMLTree( CXml * xml, QListViewItem * item )
 			FileItem->m_bIsDir = true;
 			AddFile(item,FileItem);
 			((DCFileBrowserListItem*)item1)->m_pFileItem = FileItem;
-						
+			
 			if ( g_pConfig->GetFolderSizesInLeftPane() )
 			{
-				item1->setText( 1, DCGuiUtils::GetSizeString(((DCFileBrowserListItem*)item1)->m_nBytes) );
+				item1->setText( 1, DCGuiUtils::GetSizeString( ((DCFileBrowserListItem*)item1)->m_nBytes ) );
 			}
 		}
 	}
@@ -812,10 +834,10 @@ void DCFileBrowser::ParseXMLTree( CXml * xml, QListViewItem * item )
 }
 
 /** */
-QString DCFileBrowser::CreateRemotePath( QListViewItem * item )
+QString DCFileBrowser::CreateRemotePath( Q3ListViewItem * item )
 {
 	QString s;
-	QListViewItem * pitem;
+	Q3ListViewItem * pitem;
 	
 	if ( !item )
 	{
@@ -840,18 +862,18 @@ QString DCFileBrowser::CreateRemotePath( QListViewItem * item )
 }
 
 /** */
-void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
+void DCFileBrowser::CreateMenu( Q3ListViewItem *, bool direntry, int column )
 {
-	int id,i;
+	int i;
 	QString s;
 	ulonglong size;
 	QString rootPath  = QString::null;
 	QString localName = QString::null;
-	QPtrList<QListViewItem> selitems;
-	QListViewItem * curitem;
+	QList<Q3ListViewItem*> selitems;
+	Q3ListViewItem * curitem;
 	bool hasFolderItems = false;
 	DCClient * client = 0;
-	QMap<int, DC_UserMenuCommand*> addedcommands;
+	QMap<QAction*, DC_UserMenuCommand*> addedcommands;
 	
 	if ( direntry )
 	{
@@ -861,13 +883,13 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 	{
 		i = DCGuiUtils::SelectedItems( ListView_FILES, selitems );
 		
-		QPtrListIterator<QListViewItem> plit ( selitems );
+		QListIterator<Q3ListViewItem*> it ( selitems );
 		
-		QListViewItem * cur = 0;
+		Q3ListViewItem * cur = 0;
 		
-		while ( (cur = plit.current()) != 0 )
+		while ( it.hasNext() )
 		{
-			++plit;
+			cur = it.next();
 			
 			if ( cur->text(3) == tr("Folder") )
 			{
@@ -893,53 +915,55 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 		return;
 	}
 */
-	QPopupMenu * m = new QPopupMenu(this);
+	QMenu * m = new QMenu(this);
 
-	DCMenuHandler::InsertMenu(m, emiDOWNLOAD, m_bAllowDownload);
-	DCMenuHandler::InsertMenu(m, emiDOWNLOAD_TO, m_bAllowDownload);
+	QAction * dl = DCMenuHandler::addAction(m, emiDOWNLOAD, m_bAllowDownload);
+	QAction * dl_to = DCMenuHandler::addAction(m, emiDOWNLOAD_TO, m_bAllowDownload);
 	// disable 'download as' on directories
-	DCMenuHandler::InsertMenu(m, emiDOWNLOAD_AS, m_bAllowDownload && ( !direntry && !hasFolderItems && (i == 1) ));
-	DCMenuHandler::InsertMenu(m, emiDOWNLOAD_IN, m_bAllowDownload && ( !direntry && !hasFolderItems) );
-	DCMenuHandler::InsertMenu(m, emiSEPARATOR);
-	int search_tth = m->insertItem( QIconSet(g_pIconLoader->GetPixmap(eiFILEFIND)), tr("Search by TTH") );
-	m->setItemEnabled( search_tth, m_bAllowOpen && !direntry && curitem && !(curitem->text(4).isEmpty()) );
-	DCMenuHandler::InsertMenu(m, emiSEPARATOR);
-	DCMenuHandler::InsertMenu(m, emiCOPY_COLUMN_TO_CLIPBOARD, !direntry );
-	DCMenuHandler::InsertMenu(m, emiCOPY_ROW_TO_CLIPBOARD, !direntry );
-	DCMenuHandler::InsertMenu(m, emiCOPYMAGNETLINK, (!direntry && !hasFolderItems) );
-	DCMenuHandler::InsertMenu(m, emiSEPARATOR);
-
-	client = g_pConnectionManager->GetClientForHub( m_sHubName.ascii(), m_sHubHost.ascii() );
+	QAction * dl_as = DCMenuHandler::addAction(m, emiDOWNLOAD_AS, (m_bAllowDownload && !direntry && !hasFolderItems && (i == 1) ) );
+	QAction * dl_in = DCMenuHandler::addAction(m, emiDOWNLOAD_IN, (m_bAllowDownload && !direntry && !hasFolderItems) );
+	DCMenuHandler::addAction(m, emiSEPARATOR);
+	QAction * search_tth = m->addAction( QIcon(g_pIconLoader->GetPixmap(eiFILEFIND)), tr("Search by TTH") );
+	search_tth->setEnabled( m_bAllowOpen && !direntry && curitem && !(curitem->text(4).isEmpty()) );
+	DCMenuHandler::addAction(m, emiSEPARATOR);
+	QAction * copycol = DCMenuHandler::addAction(m, emiCOPY_COLUMN_TO_CLIPBOARD, !direntry );
+	QAction * copyrow = DCMenuHandler::addAction(m, emiCOPY_ROW_TO_CLIPBOARD, !direntry );
+	QAction * copymagnet = DCMenuHandler::addAction(m, emiCOPYMAGNETLINK, (!direntry && !hasFolderItems ));
+	
+	client = g_pConnectionManager->GetClientForHub( m_sHubName.toAscii().constData(), m_sHubHost.toAscii().constData() );
 	
 	if ( client != 0 )
 	{
 		addedcommands = client->AddMenuCommands( m, euccFilelist );
 	}
 	
-	id = m->exec(QCursor::pos());
+	QAction * chosen = m->exec(QCursor::pos());
 
 	delete m;
 	
-	if ( id == emiCOPY_COLUMN_TO_CLIPBOARD )
+	if ( chosen == copycol )
 	{
 		QString s;
 
-                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
+                for ( int c = 0; c < selitems.size(); c++ )
                 {
+			curitem = selitems.at(c);
 			s += curitem->text(column);
 			s += "\n";
 		}
 
-		s = s.stripWhiteSpace();
+		s = s.trimmed();
 		QApplication::clipboard()->setText(s);
 	}
-	else if ( id == emiCOPY_ROW_TO_CLIPBOARD )
+	else if ( chosen == copyrow )
 	{
 		int idx;
 		QString s;
-		
-                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
+
+                for ( int c = 0; c < selitems.size(); c++ )
                 {
+			curitem = selitems.at(c);
+			
 			for( idx = 0; idx < ListView_FILES->columns(); idx++ )
 			{
 				s += curitem->text(idx);
@@ -949,16 +973,18 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 			s += "\n";
 		}
 
-		s = s.stripWhiteSpace();
+		s = s.trimmed();
 		QApplication::clipboard()->setText(s);
 	}
-	else if ( id == emiCOPYMAGNETLINK )
+	else if ( chosen == copymagnet )
 	{
 		//magnet:?xt=urn:tree:tiger:EOSA5AGTL5SD3VWCF3R2OH2WMGXV3S3R7SYN4YA&xl=708780032&dn=FC-6-i386-disc1.iso
 		QString all;
 		
-		for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
+		for ( int c = 0; c < selitems.size(); c++ )
 		{
+			curitem = selitems.at(c);
+			
 			// add tth
 			all += "magnet:?xt=urn:tree:tiger:";
 			all += curitem->text(4);
@@ -969,30 +995,27 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 			
 			// add filename
 			all += "&dn=";
-			QString filename = curitem->text(0);
 			
-			QUrl::encode(filename);
+			QString filename = QString(QUrl::toPercentEncoding(curitem->text(0)));
 			
 			//fixme DC++ replaces spaces with + character
 			//filename = filename.replace(' ','+');
 			
 			filename.replace("%20","+");
-			
 			all += filename;
 			
 			all += "\n";
 		}
 		
 		// remove trailing "\n"
-		all = all.stripWhiteSpace();
+		all = all.trimmed();
 		
 		QApplication::clipboard()->setText(all);
 	}
 	// Check if the user wants a non-default destination
-	else if ( id == emiDOWNLOAD_TO )
+	else if ( chosen == dl_to )
 	{
-		rootPath = QFileDialog::getExistingDirectory(QString(), this, "dldia",
-                                                     tr("Select destination"));
+		rootPath = QFileDialog::getExistingDirectory( this, tr("Select destination") );
 		if ( rootPath.isEmpty() )
 		{
 			// If the user cancel, then we don't download
@@ -1000,17 +1023,16 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 		}
 	}
 	// Should the file be renamed
-	else if(id == emiDOWNLOAD_AS)
+	else if ( chosen == dl_as )
 	{
-		rootPath = QFileDialog::getSaveFileName(selitems.first()->text(0), QString(), this, "dldia",
-                                                tr("Select a filename"));
+		rootPath = QFileDialog::getSaveFileName( this, tr("Select a filename"), selitems.first()->text(0) );
 		if ( rootPath.isEmpty() )
 		{
 			return;
 		}
 
 		QFileInfo fi(rootPath);
-		rootPath = fi.dirPath();
+		rootPath = fi.path();
 		localName = fi.fileName();
 
 		if ( rootPath.isEmpty() || localName.isEmpty() )
@@ -1018,7 +1040,7 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 			return;
 		}
 	}
-	else if ( id == emiDOWNLOAD_IN )
+	else if ( chosen == dl_in )
 	{
 		QString localrootpath;
 		QString localname;
@@ -1032,8 +1054,10 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 		QString tth = curitem->text(4);
 
 		// all files need equal size and TTH
-                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
+                for ( int c = 0; c < selitems.size(); c++ )
                 {
+			curitem = selitems.at(c);
+			
 			if ( size != ((DC_QListViewItem *)curitem)->myvalue )
 				return;
 			if ( tth != curitem->text(4) )
@@ -1046,8 +1070,10 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 		}
 
 		QString remotePathAndFile;
-                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
+                for ( int c = 0; c < selitems.size(); c++ )
                 {
+			curitem = selitems.at(c);
+			
 			s = curitem->text(0);
 			
 			// get the full remote path
@@ -1058,28 +1084,27 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 			remotePathAndFile += s;
 					    
 			// add transfer to the waitlist
-			DCFileTool::CheckFile( this, m_sNick.ascii(), m_sHubName.ascii(), m_sHubHost.ascii(),
-				  remotePathAndFile.ascii(), localname.ascii(), localpath.ascii(), localrootpath.ascii(), eltFILE,
-		  		  size, curitem->text(4).ascii(), true);
+			DCFileTool::CheckFile( this, m_sNick.toAscii().constData(), m_sHubName.toAscii().constData(), m_sHubHost.toAscii().constData(),
+				  remotePathAndFile.toAscii().constData(), localname.toAscii().constData(), localpath.toAscii().constData(), localrootpath.toAscii().constData(), eltFILE,
+		  		  size, curitem->text(4).toAscii().constData(), true);
 		}
 	}
-	else if ( id == search_tth )
+	else if ( chosen == search_tth )
 	{
-		DCHubSearch * hubsearch = new DCHubSearch( g_pConnectionManager->GetWorkspace() );
+		DCHubSearch * hubsearch = new DCHubSearch( g_pConnectionManager->GetMdiArea() );
 		hubsearch->SetSearchForFile( curitem->text(4), eftHASH );
 		hubsearch->show();
 		hubsearch->StartSearchWithPrompt();
 	}
-	else if ( addedcommands.contains( id ) )
+	else if ( addedcommands.contains( chosen ) )
 	{
-		DC_UserMenuCommand * umc = addedcommands[ id ];
+		DC_UserMenuCommand * umc = addedcommands[ chosen ];
 		
 		QString usercommand = umc->m_sCommand;
 		QString origUserCommand = usercommand;
 		QString fullfilename, filesize, filesizeshort, filetth, filetype;
 		
-		int c = 0;
-		for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
+		for ( int c = 0; c < selitems.size(); c++ )
 		{
 			// filelist browsers are only for the one nick
 			if ( (umc->m_nType == euctRawOnce) && (c > 0) )
@@ -1142,18 +1167,18 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 			
 			usercommand.replace( "%[type]", filetype );
 			
-			//client->m_pHubChat->AddStatus( usercommand.ascii() );
-			client->SendString( usercommand.ascii() );
-			
-			c++;
+			//client->m_pHubChat->AddStatus( usercommand.toAscii().constData() );
+			client->SendString( usercommand.toAscii().constData() );
 		}
 	}
 
 	// Is this a download?
-	if ( (id == emiDOWNLOAD) || (id == emiDOWNLOAD_TO) || (id == emiDOWNLOAD_AS) )
+	if ( (chosen == dl) || (chosen == dl_to) || (chosen == dl_as) )
 	{
-                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
+                for ( int c = 0; c < selitems.size(); c++ )
                 {
+			curitem = selitems.at(c);
+			
 			// directory download from left pane
 			if ( direntry )
 			{
@@ -1163,7 +1188,7 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 			}
 			else if ( curitem->text(3) == tr("Folder") ) // directory download from right pane
 			{
-				QListViewItem * diritem = GetDirItem( curitem );
+				Q3ListViewItem * diritem = GetDirItem( curitem );
 				
 				if ( diritem == 0 )
 				{
@@ -1173,7 +1198,7 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 				{
 					DownloadPath( rootPath, diritem->text(0), QString::null, diritem );
 				}
-			}
+ 			}
 			// Single file download
 			else
 			{
@@ -1181,7 +1206,7 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 				QString remoteName;
 				
 				// set localname
-				if ( id != emiDOWNLOAD_AS )
+				if ( chosen != dl_as )
 				{
 					localName = curitem->text(0);
 				}
@@ -1198,7 +1223,7 @@ void DCFileBrowser::CreateMenu( QListViewItem *, bool direntry, int column )
 }
 
 /** */
-void DCFileBrowser::DownloadPath( QString rootPath, QString localPath, QString localName, QListViewItem * item )
+void DCFileBrowser::DownloadPath( QString rootPath, QString localPath, QString localName, Q3ListViewItem * item )
 {
 	QString remotePath;
 	DCFileItem * FileItem = 0;
@@ -1213,11 +1238,13 @@ void DCFileBrowser::DownloadPath( QString rootPath, QString localPath, QString l
 	// get the full remote path
 	remotePath = CreateRemotePath(item);
 	
-	for ( FileItem = ((DCFileBrowserListItem*)item)->m_pFileList.first(); FileItem; FileItem = ((DCFileBrowserListItem*)item)->m_pFileList.next() )
+	for ( int i = 0; i < ((DCFileBrowserListItem*)item)->m_pFileList.size(); i++ )
 	{
+		FileItem = ((DCFileBrowserListItem*)item)->m_pFileList.at(i);
+		
 		if ( FileItem->m_bIsDir == false )
 		{
-		    	remoteFile = FileItem->m_sName;
+			remoteFile = FileItem->m_sName;
 			DownloadFile( rootPath, localPath, localName, remotePath, 
 				remoteFile, FileItem->m_nSize, FileItem->m_sHash );
 		}
@@ -1242,25 +1269,25 @@ void DCFileBrowser::DownloadFile( QString rootPath, QString localPath, QString l
 	// append filename to the remote path
 	remotePath += "\\";
 	remotePath += remoteName;
-			    
+		    
 	if ( localName == QString::null )
 	{
-		    localName = remoteName;
+		localName = remoteName;
 	}
 
 	
-	DCFileTool::CheckFile(this, m_sNick.ascii(),
-		m_sHubName.ascii(),
-		m_sHubHost.ascii(),
-		remotePath.ascii(),
-		(const char*)localName.ascii(),
-		(const char*)localPath.ascii(),
-		(const char*)rootPath.ascii(), eltFILE,
-		size, hash.ascii() );
+	DCFileTool::CheckFile(this, m_sNick.toAscii().constData(),
+		m_sHubName.toAscii().constData(),
+		m_sHubHost.toAscii().constData(),
+		remotePath.toAscii().constData(),
+		localName.toAscii().constData(),
+		localPath.toAscii().constData(),
+		rootPath.toAscii().constData(), eltFILE,
+		size, hash.toAscii().constData() );
 }
 
 /** Double click to download the item, copied from DCFileBrowser::CreateMenu */
-void DCFileBrowser::slotItemDoubleClicked( QListViewItem * item, const QPoint &, int )
+void DCFileBrowser::slotItemDoubleClicked( Q3ListViewItem * item, const QPoint &, int )
 {
 	QString rootPath = QString::null;
 	QString localName = QString::null;
@@ -1270,7 +1297,7 @@ void DCFileBrowser::slotItemDoubleClicked( QListViewItem * item, const QPoint &,
 		return;
 	}
 	
-	QListViewItem * diritem = GetDirItem( item );
+	Q3ListViewItem * diritem = GetDirItem( item );
 	
 	if ( diritem == 0 )
 	{
@@ -1281,10 +1308,10 @@ void DCFileBrowser::slotItemDoubleClicked( QListViewItem * item, const QPoint &,
 	if ( item->text(3) == tr("Folder") )
 	{
 		// we cannot open the folder since that would delete the existing
-		// QListViewItems, which is not allowed in this slot and causes
+		// Q3ListViewItems, which is not allowed in this slot and causes
 		// random crashes
 		
-		/* QListViewItem * diritem = ListView_DIRECTORY->currentItem();
+		/* Q3ListViewItem * diritem = ListView_DIRECTORY->currentItem();
 		
 		if ( diritem == 0 )
 		{
@@ -1315,7 +1342,7 @@ void DCFileBrowser::slotItemDoubleClicked( QListViewItem * item, const QPoint &,
 				this,
 				tr("Download contents?"),
 				tr("Download the contents of \"") + diritem->text(0) + "\" ?",
-				QMessageBox::Yes,
+				QMessageBox::Yes | QMessageBox::No,
 				QMessageBox::No
 			);
 			
@@ -1340,22 +1367,27 @@ void DCFileBrowser::slotItemDoubleClicked( QListViewItem * item, const QPoint &,
 }
 
 /** Only needed for non-xml filelists */
-ulonglong DCFileBrowser::CalcDirSize( QListViewItem * item )
+ulonglong DCFileBrowser::CalcDirSize( Q3ListViewItem * item )
 {
 	ulonglong total = 0;
 
 	DCFileBrowserListItem * cur = (DCFileBrowserListItem*) item;
 	
-	QPtrListIterator<DCFileItem> it( cur->m_pFileList );
-	for ( ; it.current(); ++it)
+	QListIterator<DCFileItem*> it( cur->m_pFileList );
+	
+	DCFileItem * current = 0;
+	
+	while ( it.hasNext())
 	{
-		if ( it.current()->m_bIsDir )
+		current = it.next();
+		
+		if ( current->m_bIsDir )
 		{
-			QListViewItem * child = item->firstChild();
+			Q3ListViewItem * child = item->firstChild();
 			
 			while ( child )
 			{
-				if ( child->text(0) == it.current()->m_sName )
+				if ( child->text(0) == current->m_sName )
 				{
 					total += CalcDirSize( child );
 					break;
@@ -1366,7 +1398,7 @@ ulonglong DCFileBrowser::CalcDirSize( QListViewItem * item )
 		}
 		else
 		{
-			total += it.current()->m_nSize;
+			total += current->m_nSize;
 		}
 	}
 	
@@ -1383,21 +1415,21 @@ ulonglong DCFileBrowser::CalcDirSize( QListViewItem * item )
 		cur->setText( 1, DCGuiUtils::GetSizeString( total ) );
 	}
 	
-	//printf( "CalcDirSize for %s = %lld\n", item->text(0).ascii(), total );
+	//printf( "CalcDirSize for %s = %lld\n", item->text(0).toAscii().constData(), total );
 	
 	return total;
 }
 
 /** */
-QListViewItem * DCFileBrowser::GetDirItem( QListViewItem * item )
+Q3ListViewItem * DCFileBrowser::GetDirItem( Q3ListViewItem * item )
 {
 	if ( item == 0 )
 	{
 		return 0;
 	}
 
-	QListViewItem * diritem = ((DC_FBListViewItem*)item)->pDirItem;
-
+	Q3ListViewItem * diritem = ((DC_FBListViewItem*)item)->pDirItem;
+	
 	if ( item->text(3) != tr("Folder") )
 	{
 		return diritem;
@@ -1448,11 +1480,11 @@ bool DCFileBrowser::DownloadDirectory( QString dir )
 {
 	bool res = false;
 	bool found = false;
-	QStringList dirs = QStringList::split( "\\", dir, false );
+	QStringList dirs = dir.split( "\\", QString::SkipEmptyParts );
 	
 	qApp->processEvents();
 	
-	QListViewItem * curitem = ListView_DIRECTORY->firstChild();
+	Q3ListViewItem * curitem = ListView_DIRECTORY->firstChild();
 	
 	if ( !curitem )
 	{
@@ -1496,13 +1528,13 @@ bool DCFileBrowser::DownloadDirectory( QString dir )
 /** */
 void DCFileBrowser::JumpTo( QString target )
 {
-	QStringList parts = QStringList::split( "\\", target, false );
+	QStringList parts = target.split( "\\", QString::SkipEmptyParts );
 	bool found = false;
 	
 	qApp->processEvents();
 	
-	QListViewItem * curitem = ListView_DIRECTORY->firstChild();
-	QListViewItem * parent;
+	Q3ListViewItem * curitem = ListView_DIRECTORY->firstChild();
+	Q3ListViewItem * parent;
 	
 	QStringList::const_iterator it = parts.constBegin();
 	for ( ; it != parts.constEnd(); ++it )
@@ -1570,7 +1602,7 @@ void DCFileBrowser::slotGotoUser()
 	}
 	else
 	{
-		client = g_pConnectionManager->GetClientForHub( m_sHubName.ascii(), m_sHubHost.ascii() );
+		client = g_pConnectionManager->GetClientForHub( m_sHubName.toAscii().constData(), m_sHubHost.toAscii().constData() );
 	}
 	
 	if ( client )
